@@ -1,4 +1,3 @@
-
 /*!
  * \file	ast.c
  * \brief	SCCP PBX Asterisk Wrapper Class
@@ -9,8 +8,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-08-04 16:45:09 +0200 (Mon, 04 Aug 2014) $
- * $Revision: 5624 $  
+ * $Date: 2014-11-05 16:21:05 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5789 $  
  */
 #include <config.h>
 #include "../../common.h"
@@ -21,7 +20,7 @@
 #include "../../sccp_socket.h"
 #include "../../sccp_pbx.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5624 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5789 $")
 
 /*
  * \brief itterate through locked pbx channels
@@ -455,15 +454,7 @@ static boolean_t sccp_wrapper_asterisk_carefullHangup(sccp_channel_t *c)
 		pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) processing hangup request, using carefull version. Standby.\n", pbx_channel_name(pbx_channel));
 		if (!pbx_check_hangup(pbx_channel)) {
 			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Channel still active.\n", pbx_channel_name(pbx_channel));
-//			if (!pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_IN_AUTOLOOP) && !pbx_channel_pbx(pbx_channel)) {
-//				pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Has no PBX -> ast_hangup.\n", pbx_channel_name(pbx_channel));
-//				ast_hangup(pbx_channel);
-//				res = TRUE;
-//			} else {
-//				pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Has PBX -> ast_queue_hangup.\n", pbx_channel_name(pbx_channel));
-//				res = ast_queue_hangup(pbx_channel) ? FALSE : TRUE;
-//			}
-			if (pbx_channel_pbx(pbx_channel) || (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING) == 0) || pbx_channel_state(pbx_channel) == AST_STATE_UP) {
+			if (pbx_channel_pbx(pbx_channel) || pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING) || pbx_channel_state(pbx_channel) == AST_STATE_UP) {
 				pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Has PBX -> ast_queue_hangup.\n", pbx_channel_name(pbx_channel));
 				res = ast_queue_hangup(pbx_channel) ? FALSE : TRUE;
 			} else {
@@ -514,9 +505,12 @@ boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t *c)
 	if (channel) {
 		PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
 		sccp_channel_stop_and_deny_scheduled_tasks(channel);
-
+		
 		channel->hangupRequest = sccp_wrapper_asterisk_carefullHangup;
 		if (!pbx_check_hangup(pbx_channel)) {
+			if (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING)) {
+				return sccp_wrapper_asterisk_requestQueueHangup(channel);
+			}
 			ast_hangup(pbx_channel);
 			res = TRUE;
 		} else {
@@ -550,6 +544,26 @@ int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcna
 
 		} else if (!strcasecmp(args, "codec")) {
 			res = sccp_channel_setPreferredCodec(c, value);
+		} else if (!strcasecmp(args, "CallingParty")) {
+			char *num, *name;
+			pbx_callerid_parse((char *)value, &name, &num);
+			sccp_channel_set_callingparty(c, name, num);
+			sccp_channel_display_callInfo(c);
+		} else if (!strcasecmp(args, "CalledParty")) {
+			char *num, *name;
+			pbx_callerid_parse((char *)value, &name, &num);
+			sccp_channel_set_calledparty(c, name, num);
+			sccp_channel_display_callInfo(c);
+		} else if (!strcasecmp(args, "OriginalCallingParty")) {
+			char *num, *name;
+			pbx_callerid_parse((char *)value, &name, &num);
+			sccp_channel_set_originalCallingparty(c, name, num);
+			sccp_channel_display_callInfo(c);
+		} else if (!strcasecmp(args, "OriginalCalledParty")) {
+			char *num, *name;
+			pbx_callerid_parse((char *)value, &name, &num);
+			sccp_channel_set_originalCalledparty(c, name, num);
+			sccp_channel_display_callInfo(c);
 		} else if (!strcasecmp(args, "microphone")) {
 			if (!value || sccp_strlen_zero(value) || !sccp_true(value)) {
 				c->setMicrophone(c, FALSE);
@@ -661,6 +675,8 @@ void sccp_asterisk_redirectedUpdate(sccp_channel_t * channel, const void *data,
 	sccp_copy_string(channel->callInfo.lastRedirectingPartyNumber, ast->cid.cid_rdnis ? ast->cid.cid_rdnis : "", sizeof(channel->callInfo.lastRedirectingPartyNumber));
 	channel->callInfo.lastRedirectingParty_valid = 1;
 #endif
+	channel->callInfo.originalCdpnRedirectReason = channel->callInfo.lastRedirectingReason;
+	channel->callInfo.lastRedirectingReason = 4;	// need to figure out these codes
 
 	sccp_channel_send_callinfo2(channel);
 }
@@ -809,6 +825,10 @@ int sccp_wrapper_sendDigits(const sccp_channel_t * channel, const char *digits)
 		pbx_log(LOG_WARNING, "No channel to send digits to\n");
 		return 0;
 	}
+	if (!digits || sccp_strlen_zero(digits)) {
+		pbx_log(LOG_WARNING, "No digits to send\n");
+		return 0;
+	}
 	//ast_channel_undefer_dtmf(channel->owner);
 	PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
 	int i;
@@ -831,8 +851,7 @@ int sccp_wrapper_sendDigits(const sccp_channel_t * channel, const char *digits)
 
 int sccp_wrapper_sendDigit(const sccp_channel_t * channel, const char digit)
 {
-	char digits[3] = "\0\0";
-	digits[0] = digit;
+	const char digits[] = { digit, '\0' };
 	sccp_log((DEBUGCAT_HIGH)) (VERBOSE_PREFIX_3 "%s: got a single digit '%c' -> '%s'\n", channel->currentDeviceId, digit, digits);
 	return sccp_wrapper_sendDigits(channel, digits);
 }
@@ -854,9 +873,10 @@ static void *sccp_asterisk_doPickupThread(void *data) {
 
 static int sccp_asterisk_doPickup(PBX_CHANNEL_TYPE *pbx_channel) {
 	pthread_t threadid;
-
-
-	pbx_channel_ref(pbx_channel);
+	
+	if (!pbx_channel || !(pbx_channel_ref(pbx_channel) > 0)) {
+		return FALSE;
+	}
 	if (ast_pthread_create_detached_background(&threadid, NULL, sccp_asterisk_doPickupThread, pbx_channel)) {
 		pbx_log(LOG_ERROR, "Unable to start Group pickup thread on channel %s\n", pbx_channel_name(pbx_channel));
 		pbx_channel_unref(pbx_channel);
@@ -917,5 +937,4 @@ enum ast_pbx_result pbx_pbx_start (PBX_CHANNEL_TYPE *pbx_channel) {
 EXIT:
 	return res;
 }
- 
 // kate: indent-width 4; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off;
