@@ -8,8 +8,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-07-02 19:24:24 +0200 (Wed, 02 Jul 2014) $
- * $Revision: 5558 $  
+ * $Date: 2014-11-05 16:21:05 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5789 $  
  */
 #ifndef __SCCP_PBX_WRAPPER_H
 #define __SCCP_PBX_WRAPPER_H
@@ -100,7 +100,7 @@
 #	include "ast112.h"
 #endif
 #ifdef ASTERISK_CONF_1_13
-#	include "astTrunk.h"
+#	include "ast113.h"
 #endif
 #include <asterisk/file.h>
 
@@ -127,15 +127,23 @@ static const struct skinny2pbx_codec_map {
 	{SKINNY_CODEC_G711_ALAW_56K,	AST_FORMAT_ALAW},
 	{SKINNY_CODEC_G711_ULAW_64K,	AST_FORMAT_ULAW},
 	{SKINNY_CODEC_G711_ULAW_56K,	AST_FORMAT_ULAW},
-	{SKINNY_CODEC_G723_1,		AST_FORMAT_G723_1},
-	{SKINNY_CODEC_G729,		AST_FORMAT_G729A},
-	{SKINNY_CODEC_G729_A,		AST_FORMAT_G729A},
 	{SKINNY_CODEC_GSM,		AST_FORMAT_GSM},
 	{SKINNY_CODEC_H261,		AST_FORMAT_H261},
 	{SKINNY_CODEC_H263,		AST_FORMAT_H263},
-	{SKINNY_CODEC_H263P,	 	AST_FORMAT_H263_PLUS},
 	{SKINNY_CODEC_T120,		AST_FORMAT_T140},
+#    if ASTERISK_VERSION_GROUP >= 113
+	{SKINNY_CODEC_G723_1,		AST_FORMAT_G723},
+	{SKINNY_CODEC_WIDEBAND_256K,	AST_FORMAT_SLIN16},
+	{SKINNY_CODEC_G729,		AST_FORMAT_G729},
+	{SKINNY_CODEC_G729_A,		AST_FORMAT_G729},
+	{SKINNY_CODEC_H263P,	 	AST_FORMAT_H263P},
+#    else
+	{SKINNY_CODEC_G723_1,		AST_FORMAT_G723_1},
 	{SKINNY_CODEC_WIDEBAND_256K,	AST_FORMAT_SLINEAR16},
+	{SKINNY_CODEC_G729,		AST_FORMAT_G729A},
+	{SKINNY_CODEC_G729_A,		AST_FORMAT_G729A},
+	{SKINNY_CODEC_H263P,	 	AST_FORMAT_H263_PLUS},
+#endif
 #    if ASTERISK_VERSION_NUMBER >= 10400
 	{SKINNY_CODEC_G726_32K,		AST_FORMAT_G726_AAL2},
 	{SKINNY_CODEC_G726_32K,		AST_FORMAT_G726},
@@ -378,3 +386,4 @@ int sccp_wrapper_sendDigit(const sccp_channel_t * channel, const char digit);
 #endif
 enum ast_pbx_result pbx_pbx_start (struct ast_channel *ast);
 #endif
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
