@@ -1,11 +1,11 @@
 /*!
- * \file	sccp_conference.c
+ * \file        sccp_conference.c
  * \brief       SCCP Conference for asterisk 10
- * \author	Marcello Ceschia <marcelloceschia [at] users.sorceforge.net>
- * \note	Reworked, but based on chan_sccp code.
+ * \author      Marcello Ceschia <marcelloceschia [at] users.sorceforge.net>
+ * \note        Reworked, but based on chan_sccp code.
  *
- * $Date: 2014-08-07 15:24:35 +0200 (Thu, 07 Aug 2014) $
- * $Revision: 5632 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 
 #include <config.h>
@@ -38,14 +38,14 @@
 #define sccp_participant_retain(_x) 	({ast_assert(_x != NULL);(sccp_conference_participant_t *)sccp_refcount_retain(_x, __FILE__, __LINE__, __PRETTY_FUNCTION__);})
 #define sccp_participant_release(_x) 	({ast_assert(_x != NULL);(sccp_conference_participant_t *)sccp_refcount_release(_x, __FILE__, __LINE__, __PRETTY_FUNCTION__);})
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5632 $")
-static int lastConferenceID = 99;
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+static uint32_t lastConferenceID = 99;
 static const uint32_t appID = APPID_CONFERENCE;
 
-static SCCP_LIST_HEAD (, sccp_conference_t) conferences;								/*!< our list of conferences */
+static SCCP_LIST_HEAD (, sccp_conference_t) conferences;							/*!< our list of conferences */
 
 static void *sccp_conference_thread(void *data);
-void sccp_conference_update_callInfo(sccp_channel_t * channel, PBX_CHANNEL_TYPE * pbxChannel);
+void sccp_conference_update_callInfo(sccp_channel_t * channel, PBX_CHANNEL_TYPE * pbxChannel, uint32_t conferenceID);
 void __sccp_conference_addParticipant(sccp_conference_t * conference, sccp_channel_t * participantChannel);
 int playback_to_channel(sccp_conference_participant_t * participant, const char *filename, int say_number);
 int playback_to_conference(sccp_conference_t * conference, const char *filename, int say_number);
@@ -83,7 +83,8 @@ void sccp_conference_module_stop(void)
 /*
  * \brief Cleanup after conference refcount goes to zero (refcount destroy)
  */
-static void __sccp_conference_destroy(sccp_conference_t * conference) {
+static void __sccp_conference_destroy(sccp_conference_t * conference)
+{
 	if (!conference) {
 		return;
 	}
@@ -92,7 +93,8 @@ static void __sccp_conference_destroy(sccp_conference_t * conference) {
 		sccp_log_and((DEBUGCAT_CONFERENCE + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Destroying conference playback channel\n", conference->id);
 #if ASTERISK_VERSION_GROUP < 112
 		PBX_CHANNEL_TYPE *underlying_channel = NULL;
-		underlying_channel = PBX(get_bridged_channel)(conference->playback_channel);
+
+		underlying_channel = PBX(get_bridged_channel) (conference->playback_channel);
 		pbx_hangup(underlying_channel);
 		pbx_hangup(conference->playback_channel);
 		pbx_channel_unref(underlying_channel);
@@ -131,18 +133,15 @@ static void __sccp_conference_participant_destroy(sccp_conference_participant_t
 	pbx_bridge_features_cleanup(&participant->features);
 #ifdef CS_MANAGER_EVENTS
 	if (GLOB(callevents)) {
-/*
-		manager_event(EVENT_FLAG_CALL, "SCCPConfLeave", "ConfId: %d\r\n" "PartId: %d\r\n" "Channel: %s\r\n" "Uniqueid: %s\r\n", 
-			participant->conference ? participant->conference->id : -1, 
-			participant->id, 
-			participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL", 
-			participant->conferenceBridgePeer ? pbx_channel_uniqueid(participant->conferenceBridgePeer) : "NULL"
-		);
-*/
-		manager_event(EVENT_FLAG_CALL, "SCCPConfLeave", "ConfId: %d\r\n" "PartId: %d\r\n", 
-			participant->conference ? participant->conference->id : -1, 
-			participant->id
-		);
+		/*
+		   manager_event(EVENT_FLAG_CALL, "SCCPConfLeave", "ConfId: %d\r\n" "PartId: %d\r\n" "Channel: %s\r\n" "Uniqueid: %s\r\n", 
+		   participant->conference ? participant->conference->id : -1, 
+		   participant->id, 
+		   participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL", 
+		   participant->conferenceBridgePeer ? pbx_channel_uniqueid(participant->conferenceBridgePeer) : "NULL"
+		   );
+		 */
+		manager_event(EVENT_FLAG_CALL, "SCCPConfLeave", "ConfId: %d\r\n" "PartId: %d\r\n", participant->conference ? participant->conference->id : -1, participant->id);
 	}
 #endif
 	if (participant->channel) {
@@ -196,7 +195,7 @@ sccp_conference_t *sccp_conference_create(sccp_device_t * device, sccp_channel_t
 	sccp_copy_string(conference->playback_language, pbx_channel_language(channel->owner), sizeof(conference->playback_language));
 	SCCP_LIST_HEAD_INIT(&conference->participants);
 
-	//bridgeCapabilities = AST_BRIDGE_CAPABILITY_1TO1MIX;							/* bridge_multiplexed */
+	//bridgeCapabilities = AST_BRIDGE_CAPABILITY_1TO1MIX;                                                   /* bridge_multiplexed */
 	bridgeCapabilities = AST_BRIDGE_CAPABILITY_MULTIMIX;							/* bridge_softmix */
 #ifdef CS_BRIDGE_CAPABILITY_MULTITHREADED
 	bridgeCapabilities |= AST_BRIDGE_CAPABILITY_MULTITHREADED;						/* bridge_softmix */
@@ -212,6 +211,12 @@ sccp_conference_t *sccp_conference_create(sccp_device_t * device, sccp_channel_t
 	ast_bridge_set_talker_src_video_mode(conference->bridge);
 #endif
 
+	if (!conference->bridge) {
+		pbx_log(LOG_WARNING, "%s: Creating conference bridge failed, cancelling conference\n", channel->designator);
+		sccp_conference_release(conference);
+		return NULL;
+	}
+
 	/*
 	   pbx_bridge_set_internal_sample_rate(conference_bridge->bridge, auto);
 	   pbx_bridge_set_mixing_interval(conference->bridge,40);
@@ -240,7 +245,7 @@ sccp_conference_t *sccp_conference_create(sccp_device_t * device, sccp_channel_t
 			channel->hangupRequest(channel);
 			return NULL;
 		}
-		channel->hangupRequest = sccp_wrapper_asterisk_requestHangup;				// moderator channel not running in a ast_pbx_start thread, but in a local thread => use hard hangup
+		channel->hangupRequest = sccp_wrapper_asterisk_requestHangup;					// moderator channel not running in a ast_pbx_start thread, but in a local thread => use hard hangup
 		sccp_conference_addParticipant_toList(conference, participant);
 		participant->channel->conference = sccp_conference_retain(conference);
 		participant->channel->conference_id = conference->id;
@@ -248,12 +253,12 @@ sccp_conference_t *sccp_conference_create(sccp_device_t * device, sccp_channel_t
 		participant->playback_announcements = device->conf_play_part_announce;
 
 		PBX(setChannelLinkedId) (participant->channel, conference->linkedid);
-		sccp_conference_update_callInfo(channel, participant->conferenceBridgePeer);
+		sccp_conference_update_callInfo(channel, participant->conferenceBridgePeer, conference->id);
 		participant->isModerator = TRUE;
-		device->conferencelist_active = device->conf_show_conflist;						// Activate conflist
-//		sccp_softkey_setSoftkeyState(device, KEYMODE_CONNCONF, SKINNY_LBL_JOIN, TRUE);
-//		sccp_softkey_setSoftkeyState(device, KEYMODE_CONNTRANS, SKINNY_LBL_JOIN, TRUE);
-		sccp_dev_set_keyset(device, sccp_device_find_index_for_line(device,channel->line->name), channel->callid, KEYMODE_CONNCONF);
+		device->conferencelist_active = device->conf_show_conflist;					// Activate conflist
+		//  sccp_softkey_setSoftkeyState(device, KEYMODE_CONNCONF, SKINNY_LBL_JOIN, TRUE);
+		//  sccp_softkey_setSoftkeyState(device, KEYMODE_CONNTRANS, SKINNY_LBL_JOIN, TRUE);
+		sccp_dev_set_keyset(device, sccp_device_find_index_for_line(device, channel->line->name), channel->callid, KEYMODE_CONNCONF);
 		pbx_builtin_setvar_int_helper(channel->owner, "__SCCP_CONFERENCE_ID", conference->id);
 		pbx_builtin_setvar_int_helper(channel->owner, "__SCCP_CONFERENCE_PARTICIPANT_ID", participant->id);
 		sccp_indicate(device, channel, SCCP_CHANNELSTATE_CONNECTEDCONFERENCE);
@@ -321,6 +326,7 @@ static void sccp_conference_connect_bridge_channels_to_participants(sccp_confere
 	AST_LIST_TRAVERSE(&bridge->channels, bridge_channel, entry) {
 		sccp_log((DEBUGCAT_HIGH + DEBUGCAT_CONFERENCE)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Bridge Channel %p.\n", conference->id, bridge_channel);
 		AUTO_RELEASE sccp_conference_participant_t *participant = sccp_conference_participant_findByPBXChannel(conference, bridge_channel->chan);
+
 		if (participant) {
 			sccp_log((DEBUGCAT_CORE + DEBUGCAT_CONFERENCE)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Connecting Bridge Channel %p to Participant %d.\n", conference->id, bridge_channel, participant->id);
 			participant->bridge_channel = bridge_channel;
@@ -376,11 +382,11 @@ static void sccp_conference_addParticipant_toList(sccp_conference_t * conference
 /*!
  * \brief Update the callinfo on the sccp channel
  */
-void sccp_conference_update_callInfo(sccp_channel_t * channel, PBX_CHANNEL_TYPE * pbxChannel)
+void sccp_conference_update_callInfo(sccp_channel_t * channel, PBX_CHANNEL_TYPE * pbxChannel, uint32_t conferenceID)
 {
 	char confstr[StationMaxNameSize] = "";
 
-	snprintf(confstr, StationMaxNameSize, "Conference %d", channel->conference_id);
+	snprintf(confstr, StationMaxNameSize, "Conference %d", conferenceID);
 
 	switch (channel->calltype) {
 		case SKINNY_CALLTYPE_INBOUND:
@@ -397,7 +403,7 @@ void sccp_conference_update_callInfo(sccp_channel_t * channel, PBX_CHANNEL_TYPE
 			channel->callInfo.calledParty_valid = 1;
 			break;
 		case skinny_calltype_LOOKUPERROR:
-			break;			 
+			break;
 	}
 
 	/* this is just a workaround to update sip and other channels also -MC */
@@ -457,11 +463,12 @@ void sccp_conference_addParticipatingChannel(sccp_conference_t * conference, scc
 		if (participant) {
 			sccp_log((DEBUGCAT_CORE + DEBUGCAT_CONFERENCE)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Adding participant %d (Channel %s)\n", conference->id, participant->id, pbx_channel_name(pbxChannel));
 
-			sccp_conference_update_callInfo(originalSCCPChannel, pbxChannel);			// Update CallerId on originalChannel before masquerade
+			sccp_conference_update_callInfo(originalSCCPChannel, pbxChannel, conference->id);	// Update CallerId on originalChannel before masquerade
 
 			// if peer is sccp then retain peer_sccp_channel
 			AUTO_RELEASE sccp_channel_t *channel = get_sccp_channel_from_pbx_channel(pbxChannel);
-			AUTO_RELEASE sccp_device_t  *device  = NULL;
+			AUTO_RELEASE sccp_device_t *device = NULL;
+
 			if (channel && (device = sccp_channel_getDevice_retained(channel))) {
 				participant->playback_announcements = device->conf_play_part_announce;
 				PBX(setChannelLinkedId) (channel, conference->linkedid);
@@ -479,8 +486,8 @@ void sccp_conference_addParticipatingChannel(sccp_conference_t * conference, scc
 					participant->channel->conference_participant_id = participant->id;
 					participant->playback_announcements = device->conf_play_part_announce;
 					sccp_indicate(device, channel, SCCP_CHANNELSTATE_CONNECTEDCONFERENCE);
-					//device->conferencelist_active = device->conf_show_conflist;			// Activate conflist on all sccp participants
-					sccp_dev_set_keyset(device, sccp_device_find_index_for_line(device,channel->line->name), channel->callid, KEYMODE_CONNCONF);
+					//device->conferencelist_active = device->conf_show_conflist;                   // Activate conflist on all sccp participants
+					sccp_dev_set_keyset(device, sccp_device_find_index_for_line(device, channel->line->name), channel->callid, KEYMODE_CONNCONF);
 				} else {									// PBX Channel
 					PBX(setPBXChannelLinkedId) (participant->conferenceBridgePeer, conference->linkedid);
 				}
@@ -488,7 +495,7 @@ void sccp_conference_addParticipatingChannel(sccp_conference_t * conference, scc
 				pbx_builtin_setvar_int_helper(participant->conferenceBridgePeer, "__SCCP_CONFERENCE_PARTICIPANT_ID", participant->id);
 #if ASTERISK_VERSION_GROUP>106
 				pbx_indicate(participant->conferenceBridgePeer, AST_CONTROL_CONNECTED_LINE);
-#endif				
+#endif
 			} else {
 				// Masq Error
 				participant = sccp_participant_release(participant);
@@ -508,6 +515,7 @@ void sccp_conference_addParticipatingChannel(sccp_conference_t * conference, scc
 static void sccp_conference_removeParticipant(sccp_conference_t * conference, sccp_conference_participant_t * participant)
 {
 	sccp_conference_participant_t *tmp_participant = NULL;
+
 	if (!conference || !participant) {
 		return;
 	}
@@ -547,16 +555,12 @@ static void sccp_conference_removeParticipant(sccp_conference_t * conference, sc
 static void *sccp_conference_thread(void *data)
 {
 	AUTO_RELEASE sccp_conference_participant_t *participant = sccp_participant_retain(data);
+
 	if (participant && participant->conference && participant->conference->bridge) {
 		sccp_log_and((DEBUGCAT_CONFERENCE + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: entering join thread.\n", participant->conference->id);
 #ifdef CS_MANAGER_EVENTS
 		if (GLOB(callevents)) {
-			manager_event(EVENT_FLAG_CALL, "SCCPConfEntered", "ConfId: %d\r\n" "PartId: %d\r\n" "Channel: %s\r\n" "Uniqueid: %s\r\n", 
-				participant->conference ? participant->conference->id : -1, 
-				participant->id, 
-				participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL", 
-				participant->conferenceBridgePeer ? pbx_channel_uniqueid(participant->conferenceBridgePeer) : "NULL"
-			);
+			manager_event(EVENT_FLAG_CALL, "SCCPConfEntered", "ConfId: %d\r\n" "PartId: %d\r\n" "Channel: %s\r\n" "Uniqueid: %s\r\n", participant->conference ? participant->conference->id : -1, participant->id, participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL", participant->conferenceBridgePeer ? pbx_channel_uniqueid(participant->conferenceBridgePeer) : "NULL");
 		}
 #endif
 		// Join the bridge
@@ -566,12 +570,7 @@ static void *sccp_conference_thread(void *data)
 		sccp_log_and((DEBUGCAT_CONFERENCE + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Leaving pbx_bridge_join: %s as %d\n", participant->conference->id, pbx_channel_name(participant->conferenceBridgePeer), participant->id);
 #ifdef CS_MANAGER_EVENTS
 		if (GLOB(callevents)) {
-			manager_event(EVENT_FLAG_CALL, "SCCPConfLeft", "ConfId: %d\r\n" "PartId: %d\r\n" "Channel: %s\r\n" "Uniqueid: %s\r\n", 
-				participant->conference ? participant->conference->id : -1, 
-				participant->id, 
-				participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL", 
-				participant->conferenceBridgePeer ? pbx_channel_uniqueid(participant->conferenceBridgePeer) : "NULL"
-			);
+			manager_event(EVENT_FLAG_CALL, "SCCPConfLeft", "ConfId: %d\r\n" "PartId: %d\r\n" "Channel: %s\r\n" "Uniqueid: %s\r\n", participant->conference ? participant->conference->id : -1, participant->id, participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL", participant->conferenceBridgePeer ? pbx_channel_uniqueid(participant->conferenceBridgePeer) : "NULL");
 		}
 #endif
 		if (participant->channel && participant->device) {
@@ -581,6 +580,8 @@ static void *sccp_conference_thread(void *data)
 		participant->pendingRemoval = TRUE;
 		sccp_conference_removeParticipant(participant->conference, participant);
 		participant->joinThread = AST_PTHREADT_NULL;
+	} else {
+		pbx_log(LOG_WARNING, "SCCP: Conference thread could not be started because of missing conference (%d), participant (%d) or conference->bridge\n", (participant && participant->conference) ? participant->conference->id : 0, participant ? participant->id : 0);
 	}
 	return NULL;
 }
@@ -606,7 +607,6 @@ void sccp_conference_start(sccp_conference_t * conference)
 #endif
 }
 
-
 /*!
  * \brief This function is called when the minimal number of occupants of a confernce is reached or when the last moderator hangs-up
  */
@@ -779,6 +779,7 @@ int playback_to_conference(sccp_conference_t * conference, const char *filename,
 
 	if (!(conference->playback_channel)) {
 		char data[14];
+
 		snprintf(data, sizeof(data), "SCCPCONF/%04d", conference->id);
 		if (!(conference->playback_channel = PBX(requestAnnouncementChannel) (AST_FORMAT_ALAW, NULL, data))) {
 			pbx_mutex_unlock(&conference->playback_lock);
@@ -828,6 +829,7 @@ int playback_to_conference(sccp_conference_t * conference, const char *filename,
 
 	if (!(conference->playback_channel)) {
 		char data[14];
+
 		snprintf(data, sizeof(data), "SCCPCONF/%04d", conference->id);
 		if (!(conference->playback_channel = PBX(requestAnnouncementChannel) (AST_FORMAT_SLINEAR, NULL, data))) {
 			pbx_mutex_unlock(&conference->playback_lock);
@@ -846,7 +848,7 @@ int playback_to_conference(sccp_conference_t * conference, const char *filename,
 		}
 
 		sccp_log_and((DEBUGCAT_CONFERENCE + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Created Playback Channel\n", conference->id);
-		underlying_channel = PBX(get_bridged_channel)(conference->playback_channel);
+		underlying_channel = PBX(get_bridged_channel) (conference->playback_channel);
 
 		// Update CDR to prevent nasty ast warning when hanging up this channel (confbridge does not set the cdr correctly)
 		pbx_cdr_start(pbx_channel_cdr(conference->playback_channel));
@@ -858,7 +860,7 @@ int playback_to_conference(sccp_conference_t * conference, const char *filename,
 		pbx_channel_unref(underlying_channel);
 	} else {
 		/* Channel was already available so we just need to add it back into the bridge */
-		underlying_channel = PBX(get_bridged_channel)(conference->playback_channel);
+		underlying_channel = PBX(get_bridged_channel) (conference->playback_channel);
 		sccp_log_and((DEBUGCAT_CONFERENCE + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Attaching '%s' to Conference\n", conference->id, pbx_channel_name(underlying_channel));
 		if (pbx_bridge_impart(conference->bridge, underlying_channel, NULL, NULL, 0)) {
 			sccp_log_and((DEBUGCAT_CONFERENCE + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_4 "SCCPCONF/%04d: Impart playback channel failed\n", conference->id);
@@ -998,18 +1000,21 @@ sccp_conference_participant_t *sccp_conference_participant_findByPBXChannel(sccp
 void sccp_conference_show_list(sccp_conference_t * conference, sccp_channel_t * c)
 {
 	int use_icon = 0;
+
 	if (!conference) {
 		pbx_log(LOG_WARNING, "SCCPCONF: No conference available to display list for\n");
 		return;
 	}
 
 	AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
+
 	if (!channel) {												// only send this list to sccp phones
 		pbx_log(LOG_WARNING, "SCCPCONF/%04d: No channel available to display conferencelist for\n", conference->id);
 		return;
 	}
 
 	AUTO_RELEASE sccp_conference_participant_t *participant = sccp_conference_participant_findByChannel(conference, channel);
+
 	if (!participant) {
 		pbx_log(LOG_WARNING, "SCCPCONF/%04d: Channel %s is not a participant in this conference\n", conference->id, pbx_channel_name(channel->owner));
 		return;
@@ -1034,7 +1039,7 @@ void sccp_conference_show_list(sccp_conference_t * conference, sccp_channel_t *
 		char xmlTmp[512] = "";
 
 		//sprintf(xmlTmp, "<CiscoIPPhoneIconMenu appId=\"%d\" onAppFocusLost=\"\" onAppFocusGained=\"\" onAppClosed=\"\">", appID);
-		if (participant->device->protocolversion >= 17) {
+		if (participant->device->protocolversion >= 15) {
 			sprintf(xmlTmp, "<CiscoIPPhoneIconFileMenu appId=\"%d\">", appID);
 			strcat(xmlStr, xmlTmp);
 			if (conference->isLocked) {
@@ -1155,7 +1160,7 @@ void sccp_conference_show_list(sccp_conference_t * conference, sccp_channel_t *
 #endif
 		}
 		// CiscoIPPhoneIconMenu Icons
-		if (participant->device->protocolversion >= 17) {
+		if (participant->device->protocolversion >= 15) {
 			strcat(xmlStr, "<IconItem><Index>0</Index><URL>Resource:Icon.Connected</URL></IconItem>");	// moderator
 			strcat(xmlStr, "<IconItem><Index>1</Index><URL>Resource:AnimatedIcon.Hold</URL></IconItem>");	// muted moderator
 			strcat(xmlStr, "<IconItem><Index>2</Index><URL>Resource:AnimatedIcon.StreamRxTx</URL></IconItem>");	// participant
@@ -1169,7 +1174,7 @@ void sccp_conference_show_list(sccp_conference_t * conference, sccp_channel_t *
 			strcat(xmlStr, "<IconItem><Index>3</Index><Height>10</Height><Width>16</Width><Depth>2</Depth><Data>000F0000C030F03CC030F03CC030F03C000FF03CFCF0F33C0C00F33CCC30F33CCC30F33C00000000</Data></IconItem>\n");	// muted participant
 		}
 
-		if (participant->device->protocolversion >= 17) {
+		if (participant->device->protocolversion >= 15) {
 			strcat(xmlStr, "</CiscoIPPhoneIconFileMenu>\n");
 		} else {
 			strcat(xmlStr, "</CiscoIPPhoneIconMenu>\n");
@@ -1205,6 +1210,7 @@ void sccp_conference_hide_list_ByDevice(sccp_device_t * device)
 	SCCP_LIST_TRAVERSE(&conferences, conference, list) {
 		if (device) {
 			AUTO_RELEASE sccp_conference_participant_t *participant = sccp_conference_participant_findByDevice(conference, device);
+
 			if (participant && participant->channel && participant->device) {
 				__sccp_conference_hide_list(participant);
 			}
@@ -1241,16 +1247,19 @@ void sccp_conference_handle_device_to_user(sccp_device_t * d, uint32_t callRefer
 		sccp_log((DEBUGCAT_CONFERENCE)) (VERBOSE_PREFIX_4 "%s: Handle DTU SoftKey Button Press for CallID %d, Transaction %d, Conference %d, Participant:%d, Action %s\n", d->id, callReference, transactionID, conferenceID, participantID, d->dtu_softkey.action);
 
 		AUTO_RELEASE sccp_conference_t *conference = sccp_conference_findByID(conferenceID);
+
 		if (!conference) {
 			pbx_log(LOG_WARNING, "%s: Conference not found\n", DEV_ID_LOG(d));
 			goto EXIT;
 		}
 		AUTO_RELEASE sccp_conference_participant_t *participant = sccp_conference_participant_findByID(conference, participantID);
+
 		if (!participant) {
 			pbx_log(LOG_WARNING, "SCCPCONF/%04d: %s: Participant not found\n", conference->id, DEV_ID_LOG(d));
 			goto EXIT;
 		}
 		AUTO_RELEASE sccp_conference_participant_t *moderator = sccp_conference_participant_findByDevice(conference, d);
+
 		if (!moderator) {
 			pbx_log(LOG_WARNING, "SCCPCONF/%04d: %s: Moderator not found\n", conference->id, DEV_ID_LOG(d));
 			goto EXIT;
@@ -1334,13 +1343,13 @@ void sccp_conference_toggle_mute_participant(sccp_conference_t * conference, scc
 		participant->features.mute = 1;
 		playback_to_channel(participant, "conf-muted", -1);
 		//if (participant->channel) {
-		//	participant->channel->setMicrophone(participant->channel, FALSE);
+		//participant->channel->setMicrophone(participant->channel, FALSE);
 		//}
 	} else {
 		participant->features.mute = 0;
 		playback_to_channel(participant, "conf-unmuted", -1);
 		//if (participant->channel) {
-		//	participant->channel->setMicrophone(participant->channel, TRUE);
+		//participant->channel->setMicrophone(participant->channel, TRUE);
 		//}
 	}
 	if (participant->channel && participant->device) {
@@ -1473,7 +1482,7 @@ void sccp_conference_invite_participant(sccp_conference_t * conference, sccp_con
 
 	if (moderator->channel && moderator->device) {
 
-		if (moderator->device->protocolversion >= 17) {
+		if (moderator->device->protocolversion >= 15) {
 			sprintf(xmlTmp, "<CiscoIPPhoneInput appId=\"%d\">\n", appID);
 		} else {
 			sprintf(xmlTmp, "<CiscoIPPhoneInput>\n");
@@ -1531,47 +1540,50 @@ char *sccp_complete_conference(OLDCONST char *line, OLDCONST char *word, int pos
 
 	switch (pos) {
 		case 2:											// action
-		{
-			for (i = 0; i < ARRAY_LEN(actions); i++) {
-				if (!strncasecmp(word, actions[i], wordlen) && ++which > state) {
-					return strdup(actions[i]);
-				}
-			}
-			break;
-		}
-		case 3:											// conferenceid
-		{
-			sccp_conference_t *conference = NULL;
-			SCCP_LIST_LOCK(&conferences);
-			SCCP_LIST_TRAVERSE(&conferences, conference, list) {
-				snprintf(tmpname, sizeof(tmpname), "%d", conference->id);
-				if (!strncasecmp(word, tmpname, wordlen) && ++which > state) {
-					ret = strdup(tmpname);
-					break;
-				}
-			}
-			SCCP_LIST_UNLOCK(&conferences);
-			break;
-		}
-		case 4:											// participantid
-		{
-			sccp_conference_participant_t *participant = NULL;
-			if (sscanf(line, "sccp conference %s %d", tmpname, &conference_id) > 0) {
-				AUTO_RELEASE sccp_conference_t *conference = sccp_conference_findByID(conference_id);
-				if (conference) {
-					SCCP_LIST_LOCK(&conference->participants);
-					SCCP_LIST_TRAVERSE(&conference->participants, participant, list) {
-						snprintf(tmpname, sizeof(tmpname), "%d", participant->id);
-						if (!strncasecmp(word, tmpname, wordlen) && ++which > state) {
-							ret = strdup(tmpname);
-							break;
-						}
+			{
+				for (i = 0; i < ARRAY_LEN(actions); i++) {
+					if (!strncasecmp(word, actions[i], wordlen) && ++which > state) {
+						return strdup(actions[i]);
 					}
-					SCCP_LIST_UNLOCK(&conference->participants);
 				}
+				break;
+			}
+		case 3:											// conferenceid
+			{
+				sccp_conference_t *conference = NULL;
+
+				SCCP_LIST_LOCK(&conferences);
+				SCCP_LIST_TRAVERSE(&conferences, conference, list) {
+					snprintf(tmpname, sizeof(tmpname), "%d", conference->id);
+					if (!strncasecmp(word, tmpname, wordlen) && ++which > state) {
+						ret = strdup(tmpname);
+						break;
+					}
+				}
+				SCCP_LIST_UNLOCK(&conferences);
+				break;
+			}
+		case 4:											// participantid
+			{
+				sccp_conference_participant_t *participant = NULL;
+
+				if (sscanf(line, "sccp conference %s %d", tmpname, &conference_id) > 0) {
+					AUTO_RELEASE sccp_conference_t *conference = sccp_conference_findByID(conference_id);
+
+					if (conference) {
+						SCCP_LIST_LOCK(&conference->participants);
+						SCCP_LIST_TRAVERSE(&conference->participants, participant, list) {
+							snprintf(tmpname, sizeof(tmpname), "%d", participant->id);
+							if (!strncasecmp(word, tmpname, wordlen) && ++which > state) {
+								ret = strdup(tmpname);
+								break;
+							}
+						}
+						SCCP_LIST_UNLOCK(&conference->participants);
+					}
+				}
+				break;
 			}
-			break;
-		}
 		default:
 			break;
 	}
@@ -1646,6 +1658,7 @@ int sccp_cli_show_conference(int fd, int *total, struct mansession *s, const str
 	}
 
 	AUTO_RELEASE sccp_conference_t *conference = sccp_conference_findByID(confid);
+
 	if (conference) {
 		sccp_conference_participant_t *participant = NULL;
 
@@ -1669,7 +1682,7 @@ int sccp_cli_show_conference(int fd, int *total, struct mansession *s, const str
 			CLI_AMI_TABLE_FIELD(Moderator,		"-11.11s",	11,	participant->isModerator ? "Yes" : "No")								\
 			CLI_AMI_TABLE_FIELD(Muted,		"-5.5s",	5,	participant->features.mute ? "Yes" : "No")								\
 			CLI_AMI_TABLE_FIELD(Announce,		"-8.8s",	8,	participant->playback_announcements ? "Yes" : "No")							\
-			CLI_AMI_TABLE_FIELD(ConfList,		"-8.8s",	8,	(participant->device && participant->device->conferencelist_active) ? "YES" : "NO") 
+			CLI_AMI_TABLE_FIELD(ConfList,		"-8.8s",	8,	(participant->device && participant->device->conferencelist_active) ? "YES" : "NO")
 
 #include "sccp_cli_table.h"
 	} else {
@@ -1713,12 +1726,14 @@ int sccp_cli_conference_command(int fd, int *total, struct mansession *s, const
 
 	if (sccp_strIsNumeric(argv[3]) && (confid = atoi(argv[3])) > 0) {
 		AUTO_RELEASE sccp_conference_t *conference = sccp_conference_findByID(confid);
+
 		if (conference) {
 			if (!strncasecmp(argv[2], "EndConf", 7)) {						// EndConf Command
 				sccp_conference_end(conference);
 			} else if (argc >= 5) {
 				if (sccp_strIsNumeric(argv[4]) && (partid = atoi(argv[4])) > 0) {
-					AUTO_RELEASE sccp_conference_participant_t *participant =  sccp_conference_participant_findByID(conference, partid);
+					AUTO_RELEASE sccp_conference_participant_t *participant = sccp_conference_participant_findByID(conference, partid);
+
 					if (participant) {
 						if (!strncasecmp(argv[2], "Kick", 4)) {				// Kick Command
 							sccp_conference_kick_participant(conference, participant);
@@ -1761,7 +1776,7 @@ int sccp_cli_conference_command(int fd, int *total, struct mansession *s, const
 		snprintf(error, sizeof(error), "At least valid ConferenceId needs to be supplied\n %s", "");
 		res = RESULT_FAILURE;
 	}
-	
+
 	if (res == RESULT_FAILURE && !sccp_strlen_zero(error)) {
 		CLI_AMI_RETURN_ERROR(fd, s, m, "%s\n", error);
 	}
