@@ -1,21 +1,20 @@
-
 /*!
- * \file	sccp_event.c
- * \brief	SCCP Event Class
- * \author	Marcello Ceschia <marcello [at] ceschia.de>
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
- * \since	2009-09-02
+ * \file        sccp_event.c
+ * \brief       SCCP Event Class
+ * \author      Marcello Ceschia <marcello [at] ceschia.de>
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
+ * \since       2009-09-02
  * 
- * $Date: 2014-06-06 11:56:09 +0200 (Fri, 06 Jun 2014) $
- * $Revision: 5473 $  
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $  
  */
 
 /*!
  * \remarks
- * Purpose:	SCCP Event
- * When to use:	Only methods directly related to sccp events should be stored in this source file.
- * Relations:	SCCP Hint
+ * Purpose:     SCCP Event
+ * When to use: Only methods directly related to sccp events should be stored in this source file.
+ * Relations:   SCCP Hint
  */
 
 #include <config.h>
@@ -24,14 +23,12 @@
 #include "sccp_device.h"
 #include "sccp_line.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5473 $")
-
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
 #define ENUMMACRO_FILE "sccp_event_enums.hh"
 #define ENUMMACRO_GENERATE ENUMMACRO_GENFUNCTION
 #include "sccp_enum_macro.h"
 #undef  ENUMMACRO_GENERATE
 #undef  ENUMMACRO_FILE
-
 void sccp_event_destroy(sccp_event_t * event);
 
 /*!
@@ -56,7 +53,7 @@ static boolean_t sccp_event_running = FALSE;
 
 void sccp_event_destroy(sccp_event_t * event)
 {
-//	pbx_log(LOG_NOTICE, "destroy event - %p type: %d: releasing held object references\n", event, event->type);
+	//pbx_log(LOG_NOTICE, "destroy event - %p type: %d: releasing held object references\n", event, event->type);
 	switch (event->type) {
 		case SCCP_EVENT_DEVICE_REGISTERED:
 		case SCCP_EVENT_DEVICE_UNREGISTERED:
@@ -89,7 +86,7 @@ void sccp_event_destroy(sccp_event_t * event)
 		case sccp_event_type_LOOKUPERROR:
 			break;
 	}
-//	pbx_log(LOG_NOTICE, "Event destroyed- %p type: %d\n", event, event->type);
+	//pbx_log(LOG_NOTICE, "Event destroyed- %p type: %d\n", event, event->type);
 }
 
 static void *sccp_event_processor(void *data)
@@ -110,7 +107,7 @@ static void *sccp_event_processor(void *data)
 	} else {
 		sccp_log((DEBUGCAT_EVENT)) (VERBOSE_PREFIX_3 "Could not retain event\n");
 	}
-	sccp_event_release(args->event);	//finale release
+	sccp_event_release(args->event);									//finale release
 	sccp_free(data);
 	return NULL;
 }
@@ -121,8 +118,8 @@ void sccp_event_module_start(void)
 
 	if (!sccp_event_running) {
 		for (i = 0; i < NUMBER_OF_EVENT_TYPES; i++) {
-			subscriptions[i].async = (sccp_event_subscriber_t *) malloc(sizeof(sccp_event_subscriber_t));
-			subscriptions[i].sync = (sccp_event_subscriber_t *) malloc(sizeof(sccp_event_subscriber_t));
+			subscriptions[i].async = (sccp_event_subscriber_t *) sccp_malloc(sizeof(sccp_event_subscriber_t));
+			subscriptions[i].sync = (sccp_event_subscriber_t *) sccp_malloc(sizeof(sccp_event_subscriber_t));
 		}
 		sccp_event_running = TRUE;
 	}
@@ -140,8 +137,8 @@ void sccp_event_module_stop(void)
 		}
 		usleep(20);											// arbitairy time for other threads to finish their business
 		for (i = 0; i < NUMBER_OF_EVENT_TYPES; i++) {
-			free(subscriptions[i].async);
-			free(subscriptions[i].sync);
+			sccp_free(subscriptions[i].async);
+			sccp_free(subscriptions[i].sync);
 		}
 	}
 }
@@ -164,7 +161,7 @@ void sccp_event_subscribe(sccp_event_type_t eventType, sccp_event_callback_t cb,
 			if (allowASyncExecution) {
 				size = subscriptions[i].aSyncSize;
 				if (size) {
-					subscriptions[i].async = (sccp_event_subscriber_t *) realloc(subscriptions[i].async, (size + 1) * sizeof(sccp_event_subscriber_t));
+					subscriptions[i].async = (sccp_event_subscriber_t *) sccp_realloc(subscriptions[i].async, (size + 1) * sizeof(sccp_event_subscriber_t));
 				}
 				subscriptions[i].async[size].callback_function = cb;
 				subscriptions[i].async[size].eventType = eventType;
@@ -172,7 +169,7 @@ void sccp_event_subscribe(sccp_event_type_t eventType, sccp_event_callback_t cb,
 			} else {
 				size = subscriptions[i].syncSize;
 				if (size) {
-					subscriptions[i].sync = (sccp_event_subscriber_t *) realloc(subscriptions[i].async, (size + 1) * sizeof(sccp_event_subscriber_t));
+					subscriptions[i].sync = (sccp_event_subscriber_t *) sccp_realloc(subscriptions[i].async, (size + 1) * sizeof(sccp_event_subscriber_t));
 				}
 				subscriptions[i].sync[size].callback_function = cb;
 				subscriptions[i].sync[size].eventType = eventType;
@@ -221,16 +218,16 @@ void sccp_event_fire(const sccp_event_t * event)
 {
 	if (event == NULL || SCCP_REF_RUNNING != sccp_refcount_isRunning() || !sccp_event_running) {
 		if (event) {
-			sccp_event_destroy((sccp_event_t *)event);
+			sccp_event_destroy((sccp_event_t *) event);
 		}
 		return;
 	}
 
 	AUTO_RELEASE sccp_event_t *e = (sccp_event_t *) sccp_refcount_object_alloc(sizeof(sccp_event_t), SCCP_REF_EVENT, sccp_event_type2str(event->type), sccp_event_destroy);
-	
+
 	if (!e) {
 		pbx_log(LOG_ERROR, "%p: Memory Allocation Error while creating sccp_event e. Exiting\n", event);
-		sccp_event_destroy((sccp_event_t *)event);
+		sccp_event_destroy((sccp_event_t *) event);
 		return;
 	}
 	// memcpy(e, event, sizeof(sccp_event_t));
@@ -290,7 +287,7 @@ void sccp_event_fire(const sccp_event_t * event)
 	if (GLOB(module_running)) {
 		if (subscriptions[i].aSyncSize > 0 && sccp_event_running) {
 			/* create thread for async subscribers */
-			struct sccp_event_aSyncEventProcessorThreadArg *arg = malloc(sizeof(struct sccp_event_aSyncEventProcessorThreadArg));
+			struct sccp_event_aSyncEventProcessorThreadArg *arg = sccp_malloc(sizeof(struct sccp_event_aSyncEventProcessorThreadArg));
 
 			if (!arg) {
 				pbx_log(LOG_ERROR, "%p: Memory Allocation Error while creating sccp_event_aSyncEventProcessorThreadArg. Skipping\n", event);
@@ -315,6 +312,7 @@ void sccp_event_fire(const sccp_event_t * event)
 
 		/* execute sync subscribers */
 		AUTO_RELEASE sccp_event_t *tmp_e = NULL;
+
 		if ((tmp_e = sccp_event_retain(e))) {
 			for (n = 0; n < subscriptions[i].syncSize && sccp_event_running; n++) {
 				if (subscriptions[i].sync[n].callback_function != NULL) {
@@ -328,6 +326,7 @@ void sccp_event_fire(const sccp_event_t * event)
 		// we are unloading. switching to synchonous mode for everything
 		sccp_log((DEBUGCAT_EVENT)) (VERBOSE_PREFIX_3 "Handling Event %p of Type %s in Forced Synchronous Mode\n", event, sccp_event_type2str(e->type));
 		AUTO_RELEASE sccp_event_t *tmp_e = NULL;
+
 		if ((tmp_e = sccp_event_retain(e))) {
 			for (n = 0; n < subscriptions[i].syncSize && sccp_event_running; n++) {
 				if (subscriptions[i].sync[n].callback_function != NULL) {
@@ -344,3 +343,5 @@ void sccp_event_fire(const sccp_event_t * event)
 		}
 	}
 }
+
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
