@@ -1,17 +1,16 @@
-
 /*!
- * \file	sccp_hint.c
- * \brief	SCCP Hint Class
- * \author	Marcello Ceschia < marcello.ceschia@users.sourceforge.net >
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- * 		See the LICENSE file at the top of the source tree.
- * \note	For more information about how does hint update works, see \ref hint_update
- * \since	2009-01-16
- * \remarks	Purpose:	SCCP Hint
- * 		When to use:	Does the business of hint status
+ * \file        sccp_hint.c
+ * \brief       SCCP Hint Class
+ * \author      Marcello Ceschia < marcello.ceschia@users.sourceforge.net >
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
+ * \note        For more information about how does hint update works, see \ref hint_update 
+ * \since       2009-01-16
+ * \remarks     Purpose:        SCCP Hint
+ *              When to use:    Does the business of hint status
  *
- * $Date: 2014-08-07 15:19:37 +0200 (Thu, 07 Aug 2014) $
- * $Revision: 5631 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 
 /*!
@@ -33,13 +32,14 @@
 #include "sccp_channel.h"
 #include "sccp_line.h"
 #include "sccp_utils.h"
+#include "sccp_indicate.h"											// only for SCCP_CHANNELSTATE_Idling
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5631 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
 
-/* ========================================================================================================================= Struct Definitions */
-/*!
- *\brief SCCP Hint Subscribing Device Structure
- */
+    /* ========================================================================================================================= Struct Definitions */
+    /*!
+     *\brief SCCP Hint Subscribing Device Structure
+     */
 typedef struct sccp_hint_SubscribingDevice sccp_hint_SubscribingDevice_t;
 typedef struct sccp_hint_list sccp_hint_list_t;
 
@@ -75,7 +75,7 @@ struct sccp_hint_lineState {
  * \brief SCCP Hint List Structure
  */
 struct sccp_hint_list {
-	pbx_mutex_t lock;											/*!< Asterisk Lock */
+	//pbx_mutex_t lock;                                                                                       /*!< Asterisk Lock */
 
 	char exten[SCCP_MAX_EXTENSION];										/*!< Extension for Hint */
 	char context[SCCP_MAX_CONTEXT];										/*!< Context for Hint */
@@ -202,6 +202,7 @@ void sccp_hint_module_stop(void)
 			SCCP_LIST_LOCK(&hint->subscribers);
 			while ((subscriber = SCCP_LIST_REMOVE_HEAD(&hint->subscribers, list))) {
 				AUTO_RELEASE sccp_device_t *device = sccp_device_retain((sccp_device_t *) subscriber->device);
+
 				if (device) {
 					subscriber->device = sccp_device_release(subscriber->device);
 					sccp_free(subscriber);
@@ -379,8 +380,9 @@ static void sccp_hint_deviceRegistered(const sccp_device_t * device)
 {
 	sccp_buttonconfig_t *config;
 	uint8_t positionOnDevice = 0;
-	
+
 	AUTO_RELEASE sccp_device_t *d = sccp_device_retain((sccp_device_t *) device);
+
 	if (d) {
 		SCCP_LIST_TRAVERSE(&d->buttonconfig, config, list) {
 			positionOnDevice++;
@@ -461,13 +463,13 @@ static void sccp_hint_addSubscription4Device(const sccp_device_t * device, const
 	}
 
 	/* skip subscribtions to already owned line */
-/*	sccp_buttonconfig_t *config;
-	SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
-		if (config->type == LINE && sccp_strcaseequals(config->button.line.name, hint_exten)) {
-			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_addSubscription4Device) Skipping Hint Registration for Line we already have this connected (%s, %s).\n", DEV_ID_LOG(device), config->button.line.name, hint_exten);
-			return;
-		}
-	}*/
+	/*      sccp_buttonconfig_t *config;
+	   SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
+	   if (config->type == LINE && sccp_strcaseequals(config->button.line.name, hint_exten)) {
+	   sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_addSubscription4Device) Skipping Hint Registration for Line we already have this connected (%s, %s).\n", DEV_ID_LOG(device), config->button.line.name, hint_exten);
+	   return;
+	   }
+	   } */
 	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_addSubscription4Device) Dialplan %s for exten: %s and context: %s\n", DEV_ID_LOG(device), hintStr, hint_exten, hint_context);
 
 	SCCP_LIST_TRAVERSE(&sccp_hint_subscriptions, hint, list) {
@@ -551,7 +553,7 @@ static sccp_hint_list_t *sccp_hint_create(char *hint_exten, char *hint_context)
 	memset(hint, 0, sizeof(sccp_hint_list_t));
 
 	SCCP_LIST_HEAD_INIT(&hint->subscribers);
-	sccp_mutex_init(&hint->lock);
+	//sccp_mutex_init(&hint->lock);
 
 	sccp_copy_string(hint->exten, hint_exten, sizeof(hint->exten));
 	sccp_copy_string(hint->context, hint_context, sizeof(hint->context));
@@ -623,6 +625,7 @@ static void sccp_hint_lineStatusChanged(sccp_line_t * line, sccp_device_t * devi
 static void sccp_hint_updateLineState(struct sccp_hint_lineState *lineState)
 {
 	AUTO_RELEASE sccp_line_t *line = sccp_line_retain(lineState->line);
+
 	if (line) {
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineState) Update Line Channel State: %s(%d)\n", line->name, sccp_channelstate2str(lineState->state), lineState->state);
 
@@ -666,6 +669,7 @@ static void sccp_hint_updateLineStateForMultipleChannels(struct sccp_hint_lineSt
 		if (SCCP_LIST_GETSIZE(&line->channels) == 1) {
 			SCCP_LIST_LOCK(&line->channels);
 			AUTO_RELEASE sccp_channel_t *channel = SCCP_LIST_FIRST(&line->channels) ? sccp_channel_retain(SCCP_LIST_FIRST(&line->channels)) : NULL;
+
 			SCCP_LIST_UNLOCK(&line->channels);
 
 			if (channel) {
@@ -721,7 +725,7 @@ static void sccp_hint_updateLineStateForMultipleChannels(struct sccp_hint_lineSt
 static void sccp_hint_updateLineStateForSingleChannel(struct sccp_hint_lineState *lineState)
 {
 	sccp_line_t *line = lineState->line;
-	uint8_t state;
+	sccp_channelstate_t state;
 
 	//boolean_t dev_privacy = FALSE;
 
@@ -730,6 +734,7 @@ static void sccp_hint_updateLineStateForSingleChannel(struct sccp_hint_lineState
 	memset(lineState->callInfo.partyNumber, 0, sizeof(lineState->callInfo.partyNumber));
 
 	AUTO_RELEASE sccp_channel_t *channel = NULL;
+
 	if (SCCP_LIST_GETSIZE(&line->channels) > 0) {
 		SCCP_LIST_LOCK(&line->channels);
 		channel = SCCP_LIST_LAST(&line->channels) ? sccp_channel_retain(SCCP_LIST_LAST(&line->channels)) : NULL;
@@ -742,9 +747,11 @@ static void sccp_hint_updateLineStateForSingleChannel(struct sccp_hint_lineState
 
 		SCCP_LIST_LOCK(&line->devices);
 		AUTO_RELEASE sccp_linedevices_t *lineDevice = SCCP_LIST_FIRST(&line->devices) ? sccp_linedevice_retain(SCCP_LIST_FIRST(&line->devices)) : NULL;
+
 		SCCP_LIST_UNLOCK(&line->devices);
 		if (lineDevice) {
 			AUTO_RELEASE sccp_device_t *device = sccp_device_retain(lineDevice->device);
+
 			if (device) {
 				if (device->dndFeature.enabled && device->dndFeature.status == SCCP_DNDMODE_REJECT) {
 					state = SCCP_CHANNELSTATE_DND;
@@ -776,11 +783,11 @@ static void sccp_hint_updateLineStateForSingleChannel(struct sccp_hint_lineState
 			case SCCP_CHANNELSTATE_HOLD:
 			case SCCP_CHANNELSTATE_CONGESTION:
 			case SCCP_CHANNELSTATE_CALLWAITING:
-			case SCCP_CHANNELSTATE_CALLTRANSFER:
-			case SCCP_CHANNELSTATE_CALLCONFERENCE:
 			case SCCP_CHANNELSTATE_CALLPARK:
 			case SCCP_CHANNELSTATE_CALLREMOTEMULTILINE:
 			case SCCP_CHANNELSTATE_INVALIDNUMBER:
+			case SCCP_CHANNELSTATE_CALLCONFERENCE:
+			case SCCP_CHANNELSTATE_CALLTRANSFER:
 				//if (dev_privacy == 0 || (dev_privacy == 1 && channel->privacy == FALSE)) {
 
 				/** set cid name/number information according to the call direction */
@@ -814,6 +821,14 @@ static void sccp_hint_updateLineStateForSingleChannel(struct sccp_hint_lineState
 						break;
 				}
 				break;
+			case SCCP_CHANNELSTATE_BLINDTRANSFER:
+			case SCCP_CHANNELSTATE_INVALIDCONFERENCE:
+			case SCCP_CHANNELSTATE_ZOMBIE:
+			case SCCP_CHANNELSTATE_PROGRESS:
+			case SCCP_CHANNELSTATE_CONNECTEDCONFERENCE:
+			case sccp_channelstate_LOOKUPERROR:
+				/* unused states */
+				break;
 		}
 
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSingleChannel) partyName: %s, partyNumber: %s\n", line->name, lineState->callInfo.partyName, lineState->callInfo.partyNumber);
@@ -842,23 +857,25 @@ static void sccp_hint_handleFeatureChangeEvent(const sccp_event_t * event)
 
 	switch (event->event.featureChanged.featureType) {
 		case SCCP_FEATURE_DND:
-		{
-			AUTO_RELEASE sccp_device_t *d = sccp_device_retain(event->event.featureChanged.device);
-			if (d) {
-				SCCP_LIST_LOCK(&d->buttonconfig);
-				SCCP_LIST_TRAVERSE(&d->buttonconfig, buttonconfig, list) {
-					if (buttonconfig->type == LINE) {
-						AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(buttonconfig->button.line.name, FALSE);
-						if (line) {
-							sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_handleFeatureChangeEvent) Notify the dnd status (%s) to asterisk for line %s\n", d->id, d->dndFeature.status ? "on" : "off", line->name);
-							sccp_hint_lineStatusChanged(line, d);
+			{
+				AUTO_RELEASE sccp_device_t *d = sccp_device_retain(event->event.featureChanged.device);
+
+				if (d) {
+					SCCP_LIST_LOCK(&d->buttonconfig);
+					SCCP_LIST_TRAVERSE(&d->buttonconfig, buttonconfig, list) {
+						if (buttonconfig->type == LINE) {
+							AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(buttonconfig->button.line.name, FALSE);
+
+							if (line) {
+								sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_handleFeatureChangeEvent) Notify the dnd status (%s) to asterisk for line %s\n", d->id, d->dndFeature.status ? "on" : "off", line->name);
+								sccp_hint_lineStatusChanged(line, d);
+							}
 						}
 					}
+					SCCP_LIST_UNLOCK(&d->buttonconfig);
 				}
-				SCCP_LIST_UNLOCK(&d->buttonconfig);
+				break;
 			}
-			break;
-		}
 		default:
 			break;
 	}
@@ -880,8 +897,6 @@ static enum ast_device_state sccp_hint_hint2DeviceState(sccp_channelstate_t stat
 			newDeviceState = AST_DEVICE_ONHOLD;
 			break;
 		case SCCP_CHANNELSTATE_BUSY:
-			newDeviceState = AST_DEVICE_BUSY;
-			break;
 		case SCCP_CHANNELSTATE_DND:
 			newDeviceState = AST_DEVICE_BUSY;
 			break;
@@ -979,11 +994,9 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 
 #ifdef CS_DYNAMIC_SPEEDDIAL
 	sccp_speed_t k;
-	char displayMessage[80];
+	char displayMessage[80] = "";
 #endif
 
-	sccp_channel_t tmpChannel;
-
 	if (!hint) {
 		pbx_log(LOG_ERROR, "SCCP: (sccp_hint_notifySubscribers) no hint provided to notifySubscribers about\n");
 		return;
@@ -996,18 +1009,10 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 
 	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_notifySubscribers) notify %u subscriber(s) of %s's state %s\n", hint->exten, SCCP_LIST_GETSIZE(&hint->subscribers), (hint->hint_dialplan) ? hint->hint_dialplan : "null", sccp_channelstate2str(hint->currentState));
 
-	/* use a temporary channel as fallback for non dynamic speeddial devices */
-	memset(&tmpChannel, 0, sizeof(sccp_channel_t));
-	sccp_copy_string(tmpChannel.callInfo.callingPartyName, hint->callInfo.partyName, sizeof(tmpChannel.callInfo.callingPartyName));
-	sccp_copy_string(tmpChannel.callInfo.calledPartyName, hint->callInfo.partyName, sizeof(tmpChannel.callInfo.calledPartyName));
-	sccp_copy_string(tmpChannel.callInfo.callingPartyNumber, hint->callInfo.partyNumber, sizeof(tmpChannel.callInfo.callingPartyNumber));
-	sccp_copy_string(tmpChannel.callInfo.calledPartyNumber, hint->callInfo.partyNumber, sizeof(tmpChannel.callInfo.calledPartyNumber));
-	tmpChannel.calltype = (hint->callInfo.calltype == SKINNY_CALLTYPE_OUTBOUND) ? SKINNY_CALLTYPE_OUTBOUND : SKINNY_CALLTYPE_INBOUND;
-	/* done */
-
 	SCCP_LIST_LOCK(&hint->subscribers);
 	SCCP_LIST_TRAVERSE(&hint->subscribers, subscriber, list) {
 		AUTO_RELEASE sccp_device_t *d = sccp_device_retain((sccp_device_t *) subscriber->device);
+
 		if (d) {
 			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) notify subscriber %s of %s's state %s (%d)\n", DEV_ID_LOG(d), d->id, (hint->hint_dialplan) ? hint->hint_dialplan : "null", sccp_channelstate2str(hint->currentState), hint->currentState);
 #ifdef CS_DYNAMIC_SPEEDDIAL
@@ -1110,7 +1115,6 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 					case SCCP_CHANNELSTATE_BUSY:
 					case SCCP_CHANNELSTATE_HOLD:
 					case SCCP_CHANNELSTATE_CALLWAITING:
-					case SCCP_CHANNELSTATE_CALLTRANSFER:
 					case SCCP_CHANNELSTATE_CALLPARK:
 					case SCCP_CHANNELSTATE_PROCEED:
 					case SCCP_CHANNELSTATE_CALLREMOTEMULTILINE:
@@ -1118,13 +1122,14 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 					case SCCP_CHANNELSTATE_DIALING:
 					case SCCP_CHANNELSTATE_PROGRESS:
 					case SCCP_CHANNELSTATE_GETDIGITS:
-					case SCCP_CHANNELSTATE_CALLCONFERENCE:
 					case SCCP_CHANNELSTATE_SPEEDDIAL:
 					case SCCP_CHANNELSTATE_DIGITSFOLL:
 					case SCCP_CHANNELSTATE_INVALIDCONFERENCE:
 					case SCCP_CHANNELSTATE_CONNECTEDCONFERENCE:
 					case SCCP_CHANNELSTATE_BLINDTRANSFER:
 					case SCCP_CHANNELSTATE_DND:
+					case SCCP_CHANNELSTATE_CALLTRANSFER:
+					case SCCP_CHANNELSTATE_CALLCONFERENCE:
 						iconstate = SKINNY_CALLSTATE_CALLREMOTEMULTILINE;
 						break;
 					case sccp_channelstate_LOOKUPERROR:
@@ -1134,7 +1139,7 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 
 				if (SCCP_CHANNELSTATE_RINGING == hint->previousState) {
 					/* we send a congestion to the phone, so call will not be marked as missed call */
-					sccp_device_sendcallstate(d, subscriber->instance, 0, SCCP_CHANNELSTATE_CONGESTION, SKINNY_CALLPRIORITY_NORMAL, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
+					sccp_device_sendcallstate(d, subscriber->instance, 0, SKINNY_CALLSTATE_CONGESTION, SKINNY_CALLPRIORITY_NORMAL, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
 				}
 
 				sccp_device_sendcallstate(d, subscriber->instance, 0, iconstate, SKINNY_CALLPRIORITY_NORMAL, SKINNY_CALLINFO_VISIBILITY_DEFAULT); /** do not set visibility to COLLAPSED, this will hidde callInfo in state CALLREMOTEMULTILINE */
@@ -1142,14 +1147,23 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 				if (hint->currentState == SCCP_CHANNELSTATE_ONHOOK || hint->currentState == SCCP_CHANNELSTATE_CONGESTION) {
 					sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, subscriber->instance, SKINNY_LAMP_OFF);
 					sccp_dev_set_keyset(d, subscriber->instance, 0, KEYMODE_ONHOOK);
-					
-				} else if (hint->currentState == SCCP_CHANNELSTATE_RINGING && d->allowRinginNotification ) {
+
+				} else if (hint->currentState == SCCP_CHANNELSTATE_RINGING && d->allowRinginNotification) {
 					sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, subscriber->instance, SKINNY_LAMP_BLINK);
 					sccp_dev_set_keyset(d, subscriber->instance, 0, KEYMODE_INUSEHINT);
-					
+
 				} else {
+					/* use a temporary channel as fallback for non dynamic speeddial devices */
+					sccp_channel_t tmpChannel = { 0 };
+					sccp_copy_string(tmpChannel.callInfo.callingPartyName, hint->callInfo.partyName, sizeof(tmpChannel.callInfo.callingPartyName));
+					sccp_copy_string(tmpChannel.callInfo.calledPartyName, hint->callInfo.partyName, sizeof(tmpChannel.callInfo.calledPartyName));
+					sccp_copy_string(tmpChannel.callInfo.callingPartyNumber, hint->callInfo.partyNumber, sizeof(tmpChannel.callInfo.callingPartyNumber));
+					sccp_copy_string(tmpChannel.callInfo.calledPartyNumber, hint->callInfo.partyNumber, sizeof(tmpChannel.callInfo.calledPartyNumber));
+					tmpChannel.calltype = (hint->callInfo.calltype == SKINNY_CALLTYPE_OUTBOUND) ? SKINNY_CALLTYPE_OUTBOUND : SKINNY_CALLTYPE_INBOUND;
+					/* done */
+
 					d->protocol->sendCallInfo(d, &tmpChannel, subscriber->instance);
-					sccp_device_setLamp(d, SKINNY_STIMULUS_LINE,subscriber->instance, SKINNY_LAMP_ON);
+					sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, subscriber->instance, SKINNY_LAMP_ON);
 					sccp_dev_set_keyset(d, subscriber->instance, 0, KEYMODE_INUSEHINT);
 				}
 			}
@@ -1206,7 +1220,7 @@ static void sccp_hint_checkForDND(struct sccp_hint_lineState *lineState)
 		if (allDevicesInDND) {
 			lineState->state = SCCP_CHANNELSTATE_DND;
 		}
-	} while(0);
+	} while (0);
 
 	if (lineState->state == SCCP_CHANNELSTATE_DND) {
 		sccp_copy_string(lineState->callInfo.partyName, "DND", sizeof(lineState->callInfo.partyName));
@@ -1229,3 +1243,88 @@ sccp_channelstate_t sccp_hint_getLinestate(const char *linename, const char *dev
 	SCCP_LIST_UNLOCK(&lineStates);
 	return state;
 }
+
+/*!
+ * \brief Show Hint LineStates
+ * \param fd Fd as int   
+ * \param total Total number of lines as int
+ * \param s AMI Session 
+ * \param m Message
+ * \param argc Argc as int
+ * \param argv[] Argv[] as char
+ * \return Result as int
+ * 
+ * \called_from_asterisk
+ */
+#include <asterisk/cli.h>
+int sccp_show_hint_lineStates(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+{
+	int local_total = 0;
+
+#define CLI_AMI_TABLE_NAME Hint_LineStates
+#define CLI_AMI_TABLE_PER_ENTRY_NAME Hint_lineState
+#define CLI_AMI_TABLE_LIST_ITER_HEAD &lineStates
+#define CLI_AMI_TABLE_LIST_ITER_TYPE struct sccp_hint_lineState
+#define CLI_AMI_TABLE_LIST_ITER_VAR lineState
+#define CLI_AMI_TABLE_LIST_LOCK SCCP_LIST_LOCK
+#define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
+#define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
+#define CLI_AMI_TABLE_FIELDS 														\
+ 		CLI_AMI_TABLE_FIELD(LineName,		"-10.10s",	10,	lineState->line->name)					\
+ 		CLI_AMI_TABLE_FIELD(State,		"-22.22s",	22,	sccp_channelstate2str(lineState->state))		\
+ 		CLI_AMI_TABLE_FIELD(CallInfoNumber,	"-15.15s",	15,	lineState->callInfo.partyNumber)			\
+ 		CLI_AMI_TABLE_FIELD(CallInfoName,	"-20.20s",	20,	lineState->callInfo.partyName)				\
+ 		CLI_AMI_TABLE_FIELD(Direction,		"-10.10s",	10,	(!SCCP_CHANNELSTATE_Idling(lineState->state) && lineState->callInfo.calltype) ? skinny_calltype2str(lineState->callInfo.calltype) : "INACTIVE")
+
+#include "sccp_cli_table.h"
+
+	if (s) {
+		*total = local_total;
+	}
+	return RESULT_SUCCESS;
+}
+
+/*!
+ * \brief Show Hint Subscriptions
+ * \param fd Fd as int   
+ * \param total Total number of lines as int
+ * \param s AMI Session 
+ * \param m Message
+ * \param argc Argc as int
+ * \param argv[] Argv[] as char
+ * \return Result as int
+ * 
+ * \called_from_asterisk
+ */
+//#include <asterisk/cli.h>
+int sccp_show_hint_subscriptions(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+{
+	int local_total = 0;
+
+#define CLI_AMI_TABLE_NAME Hint_Subscriptions
+#define CLI_AMI_TABLE_PER_ENTRY_NAME Hint_Subscription
+#define CLI_AMI_TABLE_LIST_ITER_HEAD &sccp_hint_subscriptions
+#define CLI_AMI_TABLE_LIST_ITER_TYPE sccp_hint_list_t
+#define CLI_AMI_TABLE_LIST_ITER_VAR subscription
+#define CLI_AMI_TABLE_LIST_LOCK SCCP_LIST_LOCK
+#define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
+#define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
+#define CLI_AMI_TABLE_FIELDS 														\
+ 		CLI_AMI_TABLE_FIELD(Exten,		"-10.10s",	10,	subscription->exten)					\
+ 		CLI_AMI_TABLE_FIELD(Content,		"-10.10s",	10,	subscription->context)					\
+ 		CLI_AMI_TABLE_FIELD(Hint,		"-15.15s",	15,	subscription->hint_dialplan)				\
+ 		CLI_AMI_TABLE_FIELD(State,		"-22.22s",	22,	sccp_channelstate2str(subscription->currentState))	\
+ 		CLI_AMI_TABLE_FIELD(CallInfoNumber,	"-15.15s",	15,	subscription->callInfo.partyNumber)			\
+ 		CLI_AMI_TABLE_FIELD(CallInfoName,	"-20.20s",	20,	subscription->callInfo.partyName)			\
+ 		CLI_AMI_TABLE_FIELD(Direction,		"-10.10s",	10,	(!SCCP_CHANNELSTATE_Idling(subscription->currentState) && subscription->callInfo.calltype) ? skinny_calltype2str(subscription->callInfo.calltype) : "INACTIVE")	\
+ 		CLI_AMI_TABLE_FIELD(Subs,		"-4d",		4,	SCCP_LIST_GETSIZE(&subscription->subscribers))
+
+#include "sccp_cli_table.h"
+
+	if (s) {
+		*total = local_total;
+	}
+	return RESULT_SUCCESS;
+}
+
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
