@@ -1,13 +1,12 @@
-
 /*!
- * \file	sccp_mwi.c
- * \brief	SCCP Message Waiting Indicator Class
- * \author	Marcello Ceschia <marcello.ceschia [at] users.sourceforge.net>
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- * 		See the LICENSE file at the top of the source tree.
+ * \file        sccp_mwi.c
+ * \brief       SCCP Message Waiting Indicator Class
+ * \author      Marcello Ceschia <marcello.ceschia [at] users.sourceforge.net>
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-06-27 15:02:23 +0200 (Fri, 27 Jun 2014) $
- * $Revision: 5530 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 
 #include <config.h>
@@ -18,12 +17,12 @@
 #include "sccp_line.h"
 #include "sccp_utils.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5530 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
 #ifndef CS_AST_HAS_EVENT
 #define SCCP_MWI_CHECK_INTERVAL 30
 #endif
 void sccp_mwi_checkLine(sccp_line_t * line);
-void sccp_mwi_setMWILineStatus(sccp_linedevices_t *lineDevice);
+void sccp_mwi_setMWILineStatus(sccp_linedevices_t * lineDevice);
 void sccp_mwi_linecreatedEvent(const sccp_event_t * event);
 void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event);
 void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t * line);
@@ -63,11 +62,16 @@ void sccp_mwi_module_stop(void)
 		SCCP_LIST_UNLOCK(&subscription->sccp_mailboxLine);
 		SCCP_LIST_HEAD_DESTROY(&subscription->sccp_mailboxLine);
 
-#ifdef CS_AST_HAS_EVENT
+#if defined(CS_AST_HAS_EVENT)
 		/* unsubscribe asterisk event */
 		if (subscription->event_sub) {
 			pbx_event_unsubscribe(subscription->event_sub);
 		}
+#elif defined(CS_AST_HAS_STASIS) && defined(CS_EXPERIMENTAL)
+		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (sccp_mwi_module_stop) STASIS Unsubscribe\n");
+		if (subscription->event_sub) {
+			stasis_unsubscribe(subscription->event_sub);
+		}
 #else
 		subscription->schedUpdate = SCCP_SCHED_DEL(subscription->schedUpdate);
 #endif
@@ -93,6 +97,7 @@ static void sccp_mwi_updatecount(sccp_mailbox_subscriber_list_t * subscription)
 	SCCP_LIST_LOCK(&subscription->sccp_mailboxLine);
 	SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {
 		AUTO_RELEASE sccp_line_t *line = sccp_line_retain(mailboxLine->line);
+
 		if (line) {
 			sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_4 "line: %s\n", line->name);
 			sccp_linedevices_t *lineDevice = NULL;
@@ -120,7 +125,43 @@ static void sccp_mwi_updatecount(sccp_mailbox_subscriber_list_t * subscription)
 	SCCP_LIST_UNLOCK(&subscription->sccp_mailboxLine);
 }
 
-#ifdef CS_AST_HAS_EVENT
+#if defined(CS_AST_HAS_STASIS) && defined(CS_EXPERIMENTAL)
+/*!
+ * \brief Receive MWI Event from Asterisk
+ * \param event Asterisk Event
+ * \param data Asterisk Data
+ */
+
+void sccp_mwi_event(void *userdata, struct stasis_subscription *sub, struct stasis_message *msg)
+{
+	sccp_mailbox_subscriber_list_t *subscription = userdata;
+
+	pbx_log(LOG_NOTICE, "Got mwi-event\n");
+
+	if (!subscription || !sub) {
+		return;
+	}
+
+	if (msg && ast_mwi_state_type() == stasis_message_type(msg)) {
+		struct ast_mwi_state *mwi_state = stasis_message_data(msg);
+
+		subscription->previousVoicemailStatistic.newmsgs = subscription->currentVoicemailStatistic.newmsgs;
+		subscription->previousVoicemailStatistic.oldmsgs = subscription->currentVoicemailStatistic.oldmsgs;
+
+		subscription->currentVoicemailStatistic.newmsgs = mwi_state->new_msgs;
+		subscription->currentVoicemailStatistic.oldmsgs = mwi_state->old_msgs;
+
+		if (subscription->previousVoicemailStatistic.newmsgs != subscription->currentVoicemailStatistic.newmsgs) {
+			sccp_mwi_updatecount(subscription);
+		}
+
+	} else {
+		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "Received STASIS Message that did not contain mwi state\n");
+	}
+
+}
+
+#elif defined(CS_AST_HAS_EVENT)
 /*!
  * \brief Receive MWI Event from Asterisk
  * \param event Asterisk Event
@@ -133,7 +174,7 @@ void sccp_mwi_event(const struct ast_event *event, void *data)
 	pbx_log(LOG_NOTICE, "Got mwi-event\n");
 	if (!subscription || !event) {
 		return;
-        }
+	}
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "Received PBX mwi event for %s@%s\n", (subscription->mailbox) ? subscription->mailbox : "NULL", (subscription->context) ? subscription->context : "NULL");
 
 	/* for calculation store previous voicemail counts */
@@ -153,10 +194,10 @@ void sccp_mwi_event(const struct ast_event *event, void *data)
  * \brief MWI Progress
  * \param ptr Pointer to Mailbox Subscriber list Entry
  * \note only used for asterisk version without mwi event (scheduled check)
- * 
+ *
  * \warning
  *  - line->devices is not always locked
- * 
+ *
  * \called_from_asterisk
  */
 int sccp_mwi_checksubscription(const void *ptr)
@@ -165,7 +206,7 @@ int sccp_mwi_checksubscription(const void *ptr)
 
 	if (!subscription) {
 		return -1;
-        }
+	}
 	subscription->previousVoicemailStatistic.newmsgs = subscription->currentVoicemailStatistic.newmsgs;
 	subscription->previousVoicemailStatistic.oldmsgs = subscription->currentVoicemailStatistic.oldmsgs;
 
@@ -181,11 +222,12 @@ int sccp_mwi_checksubscription(const void *ptr)
 	}
 
 	/* reschedule my self */
-	if ((subscription->schedUpdate = sccp_sched_add(SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
+	if ((subscription->schedUpdate = PBX(sched_add) (SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
 		pbx_log(LOG_ERROR, "Error creating mailbox subscription.\n");
 	}
 	return 0;
 }
+
 #endif
 
 /*!
@@ -200,7 +242,7 @@ void sccp_mwi_unsubscribeMailbox(sccp_mailbox_t ** mailbox)
 }
 
 /*!
- * \brief Device Attached Event 
+ * \brief Device Attached Event
  * \param event SCCP Event
  */
 void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event)
@@ -211,10 +253,10 @@ void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event)
 	}
 
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_1 "SCCP: (mwi_deviceAttachedEvent) Get deviceAttachedEvent\n");
-	
-	sccp_linedevices_t 	*linedevice	= event->event.deviceAttached.linedevice;
-	sccp_line_t 		*line 		= linedevice->line;
-	sccp_device_t 		*device		= linedevice->device;
+
+	sccp_linedevices_t *linedevice = event->event.deviceAttached.linedevice;
+	sccp_line_t *line = linedevice->line;
+	sccp_device_t *device = linedevice->device;
 
 	if (line && device) {
 		device->voicemailStatistic.oldmsgs += line->voicemailStatistic.oldmsgs;
@@ -226,7 +268,7 @@ void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event)
 }
 
 /*!
- * \brief Line Status Changed Event 
+ * \brief Line Status Changed Event
  * \param event SCCP Event
  */
 void sccp_mwi_lineStatusChangedEvent(const sccp_event_t * event)
@@ -237,17 +279,15 @@ void sccp_mwi_lineStatusChangedEvent(const sccp_event_t * event)
 	}
 
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_1 "SCCP: (mwi_lineStatusChangedEvent) Get lineStatusChangedEvent\n");
-	if (	event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_DOWN || 
-		event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_ONHOOK ||
-		event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_RINGING) {				/* these are the only events we are interested in */
+	if (event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_DOWN || event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_ONHOOK || event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_RINGING) {	/* these are the only events we are interested in */
 		sccp_mwi_check(event->event.lineStatusChanged.optional_device);
 	}
 }
 
 /*!
- * \brief Line Created Event 
+ * \brief Line Created Event
  * \param event SCCP Event
- * 
+ *
  * \warning
  *  - line->mailboxes is not always locked
  */
@@ -277,7 +317,7 @@ void sccp_mwi_linecreatedEvent(const sccp_event_t * event)
  * \param mailbox Mailbox as char
  * \param context Mailbox Context
  * \param line SCCP Line
- * 
+ *
  * \warning
  *  - subscription->sccp_mailboxLine is not always locked
  */
@@ -288,10 +328,7 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 
 	SCCP_LIST_LOCK(&sccp_mailbox_subscriptions);
 	SCCP_LIST_TRAVERSE(&sccp_mailbox_subscriptions, subscription, list) {
-		if (strlen(mailbox) == strlen(subscription->mailbox) &&
-		    strlen(context) == strlen(subscription->context) &&
-		    !strcmp(mailbox, subscription->mailbox) &&
-		    !strcmp(context, subscription->context)) {
+		if (strlen(mailbox) == strlen(subscription->mailbox) && strlen(context) == strlen(subscription->context) && !strcmp(mailbox, subscription->mailbox) && !strcmp(context, subscription->context)) {
 			break;
 		}
 	}
@@ -316,6 +353,7 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 		SCCP_LIST_UNLOCK(&sccp_mailbox_subscriptions);
 
 		/* get initial value */
+
 #ifdef CS_AST_HAS_EVENT
 		struct ast_event *event = ast_event_get_cached(AST_EVENT_MWI,
 							       AST_EVENT_IE_MAILBOX, AST_EVENT_IE_PLTYPE_STR, subscription->mailbox,
@@ -335,7 +373,7 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 			pbx_app_inboxcount(buffer, &subscription->currentVoicemailStatistic.newmsgs, &subscription->currentVoicemailStatistic.oldmsgs);
 		}
 
-#ifdef CS_AST_HAS_EVENT
+#if defined( CS_AST_HAS_EVENT)
 		/* register asterisk event */
 		//struct pbx_event_sub *pbx_event_subscribe(enum ast_event_type event_type, ast_event_cb_t cb, char *description, void *userdata, ...);
 #if ASTERISK_VERSION_NUMBER >= 10800
@@ -346,8 +384,21 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 		if (!subscription->event_sub) {
 			pbx_log(LOG_ERROR, "SCCP: PBX MWI event could not be subscribed to for mailbox %s@%s\n", subscription->mailbox, subscription->context);
 		}
+#elif defined(CS_AST_HAS_STASIS) && defined(CS_EXPERIMENTAL)
+		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_addMailboxSubscription) Adding STASIS Subscription for mailbox %s\n", subscription->mailbox);
+		char mailbox_context[512];
+
+		sprintf(mailbox_context, "%s@%s", subscription->mailbox, subscription->context);
+
+		struct stasis_topic *mailbox_specific_topic;
+
+		mailbox_specific_topic = ast_mwi_topic(mailbox_context);
+		if (mailbox_specific_topic) {
+			subscription->event_sub = stasis_subscribe(mailbox_specific_topic, sccp_mwi_event, subscription);
+		}
 #else
-		if ((subscription->schedUpdate = sccp_sched_add(SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
+		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_addMailboxSubscription) Falling back to polling mailbox status\n");
+		if ((subscription->schedUpdate = PBX(sched_add) (SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
 			pbx_log(LOG_ERROR, "SCCP: (mwi_addMailboxSubscription) Error creating mailbox subscription.\n");
 		}
 #endif
@@ -357,7 +408,7 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 	SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {
 		if (line == mailboxLine->line) {
 			break;
-                }
+		}
 	}
 
 	if (!mailboxLine) {
@@ -411,7 +462,7 @@ void sccp_mwi_checkLine(sccp_line_t * line)
  * \brief Set MWI Line Status
  * \param lineDevice SCCP LineDevice
  */
-void sccp_mwi_setMWILineStatus(sccp_linedevices_t *lineDevice)
+void sccp_mwi_setMWILineStatus(sccp_linedevices_t * lineDevice)
 {
 	sccp_msg_t *msg;
 	sccp_line_t *l = lineDevice->line;
@@ -476,8 +527,9 @@ void sccp_mwi_check(sccp_device_t * d)
 
 	/* check if we have an active channel */
 	boolean_t hasActiveChannel = FALSE, hasRinginChannel = FALSE;
-	
+
 	AUTO_RELEASE sccp_device_t *device = sccp_device_retain(d);
+
 	if (!device) {
 		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_check) called with NULL device!\n");
 		return;
@@ -488,14 +540,17 @@ void sccp_mwi_check(sccp_device_t * d)
 	SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
 		if (config->type == LINE) {
 			AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(config->button.line.name, FALSE);
+
 			if (!line) {
 				sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: NULL line retrieved from buttonconfig!\n", DEV_ID_LOG(device));
 				continue;
 			}
 			sccp_channel_t *c = NULL;
+
 			SCCP_LIST_LOCK(&line->channels);
 			SCCP_LIST_TRAVERSE(&line->channels, c, list) {
 				AUTO_RELEASE sccp_device_t *tmpDevice = sccp_channel_getDevice_retained(c);
+
 				if (tmpDevice && tmpDevice == device) {						// We have a channel belonging to our device (no remote shared line channel)
 					if (c->state != SCCP_CHANNELSTATE_ONHOOK && c->state != SCCP_CHANNELSTATE_DOWN) {
 						hasActiveChannel = TRUE;
@@ -570,14 +625,14 @@ void sccp_mwi_check(sccp_device_t * d)
 
 /*!
  * \brief Show MWI Subscriptions
- * \param fd Fd as int   
+ * \param fd Fd as int
  * \param total Total number of lines as int
- * \param s AMI Session 
+ * \param s AMI Session
  * \param m Message
  * \param argc Argc as int
  * \param argv[] Argv[] as char
  * \return Result as int
- * 
+ *
  * \called_from_asterisk
  */
 #include <asterisk/cli.h>
@@ -626,3 +681,5 @@ int sccp_show_mwi_subscriptions(int fd, int *total, struct mansession *s, const
 	}
 	return RESULT_SUCCESS;
 }
+
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
