@@ -1,4 +1,3 @@
-
 /*!
  * \file        sccp_pbx.c
  * \brief       SCCP PBX Asterisk Wrapper Class
@@ -9,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-07-02 20:10:32 +0200 (Wed, 02 Jul 2014) $
- * $Revision: 5562 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 #ifndef __PBX_IMPL_C
 #define __PBX_IMPL_C
@@ -27,7 +26,7 @@
 #include "sccp_indicate.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5562 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
 
     /*!
      * \brief SCCP Structure to pass data to the pbx answer thread 
@@ -204,31 +203,16 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
 	SCCP_LIST_LOCK(&l->devices);
 	SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
 		/* do we have cfwd enabled? */
-		if (linedevice->cfwdAll.enabled) {
-			pbx_log(LOG_NOTICE, "%s: initialize cfwd for line %s\n", linedevice->device->id, l->name);
-			if (sccp_channel_forward(c, linedevice, linedevice->cfwdAll.number) == 0) {
-				sccp_device_sendcallstate(linedevice->device, linedevice->lineInstance, c->callid, SKINNY_CALLSTATE_INTERCOMONEWAY, SKINNY_CALLPRIORITY_NORMAL, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
-				sccp_channel_send_callinfo(linedevice->device, c);
-#ifdef CS_EXPERIMENTAL
-				if (sccp_strlen_zero(pbx_builtin_getvar_helper(c->owner, "FORWARDER_FOR"))) {
-					struct ast_var_t *variables;
-					const char *var, *val;
-					char mask[25];
-
-					ast_channel_lock(c->owner);
-					sprintf(mask, "SCCP::%d", c->callid);
-					AST_LIST_TRAVERSE(pbx_channel_varshead(c->owner), variables, entries) {
-						if ((var = ast_var_name(variables)) && (val = ast_var_value(variables)) && (!strcmp("LINKID", var)) && (strcmp(mask, val))) {
-							sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: LINKID %s\n", val);
-							pbx_builtin_setvar_helper(c->owner, "__FORWARDER_FOR", val);
-						}
-					}
-					ast_channel_unlock(c->owner);
+		if (sccp_strlen_zero(pbx_builtin_getvar_helper(c->owner, "BYPASS_CFWD"))) {
+			if (linedevice->cfwdAll.enabled || (linedevice->cfwdBusy.enabled && (linedevice->device->state != SCCP_DEVICESTATE_ONHOOK || linedevice->device->accessoryStatus.speaker))) {
+				pbx_log(LOG_NOTICE, "%s: initialize cfwd%s for line %s\n", linedevice->device->id, (linedevice->cfwdAll.enabled ? "All" : (linedevice->cfwdBusy.enabled ? "Busy" : "None")), l->name);
+				if (sccp_channel_forward(c, linedevice, linedevice->cfwdAll.enabled ? linedevice->cfwdAll.number : linedevice->cfwdBusy.number) == 0) {
+					sccp_device_sendcallstate(linedevice->device, linedevice->lineInstance, c->callid, SKINNY_CALLSTATE_INTERCOMONEWAY, SKINNY_CALLPRIORITY_NORMAL, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
+					sccp_channel_send_callinfo(linedevice->device, c);
+					isRinging = TRUE;
 				}
-#endif
-				isRinging = TRUE;
+				continue;
 			}
-			continue;
 		}
 
 		if (!linedevice->device->session) {
@@ -276,7 +260,7 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
 	SCCP_LIST_UNLOCK(&l->devices);
 
 	if (isRinging) {
-		//              sccp_channel_setSkinnyCallstate(c, SKINNY_CALLSTATE_RINGIN);
+		//  sccp_channel_setSkinnyCallstate(c, SKINNY_CALLSTATE_RINGIN);
 		sccp_channel_setChannelstate(c, SCCP_CHANNELSTATE_RINGING);
 		PBX(queue_control) (c->owner, AST_CONTROL_RINGING);
 	} else if (hasDNDParticipant) {
@@ -295,10 +279,10 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
 	}
 
 	if (cid_name) {
-		free(cid_name);
+		sccp_free(cid_name);
 	}
 	if (cid_number) {
-		free(cid_number);
+		sccp_free(cid_number);
 	}
 
 	return isRinging != TRUE;
@@ -359,7 +343,7 @@ int sccp_pbx_hangup(sccp_channel_t * channel)
 	sccp_channel_closeAllMediaTransmitAndReceive(d, c);
 
 	// removing scheduled dialing
-	c->scheduler.digittimeout = SCCP_SCHED_DEL(c->scheduler.digittimeout);
+	sccp_channel_stop_schedule_digittimout(c);
 
 	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "%s: Current channel %s-%08x state %s(%d)\n", (d) ? DEV_ID_LOG(d) : "(null)", l ? l->name : "(null)", c->callid, sccp_channelstate2str(c->state), c->state);
 
@@ -671,8 +655,7 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 	sccp_channel_updateChannelCapability(c);
 	PBX(set_nativeAudioFormats) (c, c->preferences.audio, 1);
 
-	//! \todo check locking
-	/* \todo we should remove this shit. */
+	/* can be replaced with c->designator */
 	char tmpName[StationMaxNameSize];
 
 	snprintf(tmpName, sizeof(tmpName), "SCCP/%s-%08x", l->name, c->callid);
@@ -682,6 +665,8 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 
 	// \todo: Bridge?
 	// \todo: Transfer?
+
+	/* \todo: can be done using atomic function, maybe we should not be using this method at all */
 	sccp_mutex_lock(&GLOB(usecnt_lock));
 	GLOB(usecnt)++;
 	sccp_mutex_unlock(&GLOB(usecnt_lock));
@@ -694,6 +679,57 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 	if (PBX(set_callerid_name)) {
 		PBX(set_callerid_name) (c, c->callInfo.callingPartyName);
 	}
+
+	/* call ast_channel_call_forward_set with the forward destination if this device is forwarded */
+	if (SCCP_LIST_GETSIZE(&l->devices) == 1) {
+		sccp_linedevices_t *linedevice;
+
+		SCCP_LIST_LOCK(&l->devices);
+		SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
+			if (linedevice->line == l) {
+				if (linedevice->cfwdAll.enabled) {
+					sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: ast call forward channel_set: %s\n", c->designator, linedevice->cfwdAll.number);
+					PBX(setChannelCallForward) (c, linedevice->cfwdAll.number);
+				} else if (linedevice->cfwdBusy.enabled && (linedevice->device->state != SCCP_DEVICESTATE_ONHOOK || linedevice->device->accessoryStatus.speaker)) {
+					sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: ast call forward channel_set: %s\n", c->designator, linedevice->cfwdBusy.number);
+					PBX(setChannelCallForward) (c, linedevice->cfwdBusy.number);
+				}
+				break;
+			}
+		}
+		SCCP_LIST_UNLOCK(&l->devices);
+	}
+#if 0
+	{
+
+		/* (shared line version) call ast_channel_call_forward_set if all devices for this line are forwarded. Send the first forward destination to PBX */
+		sccp_linedevices_t *linedevice;
+		int numdevices = SCCP_LIST_GETSIZE(&l->devices);
+		int numforwards = 0;
+		char cfwdnum[SCCP_MAX_EXTENSION] = "";
+
+		SCCP_LIST_LOCK(&l->devices);
+		SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
+			if (linedevice->line == l && (linedevice->cfwdAll.enabled || (linedevice->cfwdBusy.enabled && (linedevice->device->state != SCCP_DEVICESTATE_ONHOOK || linedevice->device->accessoryStatus.speaker)))
+			    ) {
+				numforwards++;
+				if (sccp_strlen_zero(cfwdnum)) {
+					if (linedevice->cfwdAll.number) {
+						sccp_copy_string(cfwdnum, linedevice->cfwdAll.number, SCCP_MAX_EXTENSION);
+					} else {
+						sccp_copy_string(cfwdnum, linedevice->cfwdBusy.number, SCCP_MAX_EXTENSION);
+					}
+				}
+			}
+		}
+		SCCP_LIST_UNLOCK(&l->devices);
+		if (numdevices == numforwards) {
+			sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: setting ast call forward channel: %s\n", c->designator, cfwdnum);
+			PBX(setChannelCallForward) (c, cfwdnum);
+		}
+	}
+#endif
+
 	/* asterisk needs the native formats bevore dialout, otherwise the next channel gets the whole AUDIO_MASK as requested format
 	 * chan_sip dont like this do sdp processing */
 	//PBX(set_nativeAudioFormats)(c, c->preferences.audio, ARRAY_LEN(c->preferences.audio));
@@ -717,12 +753,15 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
  */
 int sccp_pbx_sched_dial(const void *data)
 {
-	AUTO_RELEASE sccp_channel_t *c = sccp_channel_retain((sccp_channel_t *) data);
+	AUTO_RELEASE sccp_channel_t *c = sccp_channel_retain((sccp_channel_t *) data);				// channel already retained in data
 
-	if (c && c->owner && !PBX(getChannelPbx) (c)) {
-		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: Timeout for call '%d'. Going to dial '%s'\n", c->callid, c->dialedNumber);
-		c->scheduler.digittimeout = 0;
-		sccp_pbx_softswitch(c);
+	if (c) {
+		c->scheduler.digittimeout = -1;
+		if (c->owner && !PBX(getChannelPbx) (c)) {
+			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: Timeout for call '%d'. Going to dial '%s'\n", c->callid, c->dialedNumber);
+			sccp_pbx_softswitch(c);
+		}
+		sccp_channel_release(c);									// release scheduled dial channel retension (scheduled digit timed out)
 	}
 	return 0;
 }
@@ -739,7 +778,7 @@ sccp_extension_status_t sccp_pbx_helper(sccp_channel_t * c)
 	if (!sccp_strlen_zero(c->dialedNumber)) {
 		if (GLOB(recorddigittimeoutchar) && GLOB(digittimeoutchar) == c->dialedNumber[strlen(c->dialedNumber) - 1]) {
 			/* we finished dialing with digit timeout char */
-			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: We finished dialing with digit timeout char %s\n", c->dialedNumber);
+			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_2 "%s: We finished dialing with digit timeout char %s\n", c->designator, c->dialedNumber);
 			return SCCP_EXTENSION_EXACTMATCH;
 		}
 	}
@@ -756,14 +795,14 @@ sccp_extension_status_t sccp_pbx_helper(sccp_channel_t * c)
 
 		if (d) {
 			if (((d->overlapFeature.enabled && !extensionStatus) || (!d->overlapFeature.enabled && !extensionStatus))) {
-				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: %s Matches More\n", c->dialedNumber);
+				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: %s Matches More\n", c->designator, c->dialedNumber);
 				return SCCP_EXTENSION_MATCHMORE;
 			}
-			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: %s Match %s\n", c->dialedNumber, extensionStatus == SCCP_EXTENSION_EXACTMATCH ? "Exactly" : "More");
+			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: %s Matches %s\n", c->designator, c->dialedNumber, extensionStatus == SCCP_EXTENSION_EXACTMATCH ? "Exactly" : "More");
 		}
 		return extensionStatus;
 	}
-	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: %s Does Exists\n", c->dialedNumber);
+	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_2 "%s: %s Does Exists\n", c->designator, c->dialedNumber);
 	return SCCP_EXTENSION_NOTEXISTS;
 }
 
@@ -784,19 +823,21 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 			pbx_log(LOG_ERROR, "SCCP: (sccp_pbx_softswitch) No <channel> available. Returning from dial thread.\n");
 			goto EXIT_FUNC;
 		}
+		/* removing scheduled dialing */
+		sccp_channel_stop_schedule_digittimout(c);
 
 		/* Reset Enbloc Dial Emulation */
 		c->enbloc.deactivate = 0;
 		c->enbloc.totaldigittime = 0;
 		c->enbloc.totaldigittimesquared = 0;
-		c->enbloc.digittimeout = GLOB(digittimeout) * 1000;
+		c->enbloc.digittimeout = GLOB(digittimeout);
 
 		/* prevent softswitch from being executed twice (Pavel Troller / 15-Oct-2010) */
 		if (PBX(getChannelPbx) (c)) {
 			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "SCCP: (sccp_pbx_softswitch) PBX structure already exists. Dialing instead of starting.\n");
 			/* If there are any digits, send them instead of starting the PBX */
 			if (!sccp_strlen_zero(c->dialedNumber)) {
-				//                              sccp_pbx_senddigits(c, c->dialedNumber);
+				//sccp_pbx_senddigits(c, c->dialedNumber);
 				if (PBX(send_digits)) {
 					PBX(send_digits) (channel, c->dialedNumber);
 				}
@@ -811,9 +852,6 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 		}
 		pbx_channel = pbx_channel_ref(c->owner);
 
-		/* removing scheduled dialing */
-		c->scheduler.digittimeout = SCCP_SCHED_DEL(c->scheduler.digittimeout);
-
 		/* we should just process outbound calls, let's check calltype */
 		if (c->calltype != SKINNY_CALLTYPE_OUTBOUND) {
 			pbx_log(LOG_ERROR, "SCCP: (sccp_pbx_softswitch) This function is for outbound calls only. Exiting\n");
@@ -880,12 +918,16 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 		/* This will choose what to do */
 		switch (c->ss_action) {
 			case SCCP_SS_GETFORWARDEXTEN:
-				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Get Forward Extension\n", d->id);
-				if (!sccp_strlen_zero(shortenedNumber)) {
-					sccp_line_cfwd(l, d, c->ss_data, shortenedNumber);
+				{
+					sccp_callforward_t type = (sccp_callforward_t) c->ss_data;
+
+					sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Get Forward %s Extension\n", d->id, sccp_callforward2str(type));
+					if (!sccp_strlen_zero(shortenedNumber)) {
+						sccp_line_cfwd(l, d, type, shortenedNumber);
+					}
+					sccp_channel_endcall(c);
+					goto EXIT_FUNC;								// leave simple switch without dial
 				}
-				sccp_channel_endcall(c);
-				goto EXIT_FUNC;									// leave simple switch without dial
 #ifdef CS_SCCP_PICKUP
 			case SCCP_SS_GETPICKUPEXTEN:
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Get Pickup Extension\n", d->id);
@@ -898,9 +940,9 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 
 				if (!sccp_strlen_zero(shortenedNumber)) {
 					sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "SCCP: (sccp_pbx_softswitch) Asterisk request to pickup exten '%s'\n", shortenedNumber);
-					if (!sccp_feat_directed_pickup(c, shortenedNumber)) {
-						//                                      sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);
-					}
+					//if (!sccp_feat_directed_pickup(c, shortenedNumber)) {
+					//	sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);
+					//}
 					PBX(set_callstate) (c, AST_STATE_UP);
 				} else {
 					// without a number we can also close the call. Isn't it true ?
@@ -917,7 +959,9 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 					sccp_channel_setChannelstate(c, SCCP_CHANNELSTATE_PROCEED);
 					PBX(set_callstate) (channel, AST_STATE_UP);
 					if (!d->conference) {
-						d->conference = sccp_conference_create(d, c);
+						if (!(d->conference = sccp_conference_create(d, c))) {
+							goto EXIT_FUNC;
+						}
 						sccp_indicate(d, c, SCCP_CHANNELSTATE_CONNECTEDCONFERENCE);
 					} else {
 						pbx_log(LOG_NOTICE, "%s: There is already a conference running on this device.\n", DEV_ID_LOG(d));
@@ -1047,48 +1091,51 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 		/*! \todo DdG: Extra wait time is incurred when checking pbx_exists_extension, when a wrong number is dialed. storing extension_exists status for sccp_log use */
 		int extension_exists = SCCP_EXTENSION_NOTEXISTS;
 
-		if (!sccp_strlen_zero(shortenedNumber) && (pbx_channel && !pbx_check_hangup(pbx_channel)) &&
-		    /*  && (extension_exists = pbx_exists_extension(pbx_channel, pbx_channel_context(pbx_channel), shortenedNumber, 1, l->cid_num)) */
-		    ((extension_exists = PBX(extension_status(c)) != SCCP_EXTENSION_NOTEXISTS))
+		if (!sccp_strlen_zero(shortenedNumber) && ((extension_exists = PBX(extension_status(c)) != SCCP_EXTENSION_NOTEXISTS))
 		    ) {
-			/* found an extension, let's dial it */
-			sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) channel %s-%08x is dialing number %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
+			if (pbx_channel && !pbx_check_hangup(pbx_channel)) {
+				/* found an extension, let's dial it */
+				sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) channel %s-%08x is dialing number %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
 
-			sccp_copy_string(c->callInfo.calledPartyNumber, shortenedNumber, sizeof(c->callInfo.calledPartyNumber));
-			/* Answer dialplan command works only when in RINGING OR RING ast_state */
-			PBX(set_callstate) (c, AST_STATE_RING);
+				sccp_copy_string(c->callInfo.calledPartyNumber, shortenedNumber, sizeof(c->callInfo.calledPartyNumber));
+				/* Answer dialplan command works only when in RINGING OR RING ast_state */
+				PBX(set_callstate) (c, AST_STATE_RING);
 
-			enum ast_pbx_result pbxStartResult = pbx_pbx_start(pbx_channel);
+				enum ast_pbx_result pbxStartResult = pbx_pbx_start(pbx_channel);
 
-			/* \todo replace AST_PBX enum using pbx_impl wrapper enum */
-			switch (pbxStartResult) {
-				case AST_PBX_FAILED:
-					pbx_log(LOG_ERROR, "%s: (sccp_pbx_softswitch) channel %s-%08x failed to start new thread to dial %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
-					/* \todo change indicate to something more suitable */
-					sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);
-					c->scheduler.hangup = sccp_sched_add(15000, sccp_channel_sched_endcall_by_callid, &c->callid);
-					break;
-				case AST_PBX_CALL_LIMIT:
-					pbx_log(LOG_WARNING, "%s: (sccp_pbx_softswitch) call limit reached for channel %s-%08x failed to start new thread to dial %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
-					sccp_indicate(d, c, SCCP_CHANNELSTATE_CONGESTION);
-					c->scheduler.hangup = sccp_sched_add(15000, sccp_channel_sched_endcall_by_callid, &c->callid);
-					break;
-				case AST_PBX_SUCCESS:
-					sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) pbx started\n", DEV_ID_LOG(d));
+				/* \todo replace AST_PBX enum using pbx_impl wrapper enum */
+				switch (pbxStartResult) {
+					case AST_PBX_FAILED:
+						pbx_log(LOG_ERROR, "%s: (sccp_pbx_softswitch) channel %s-%08x failed to start new thread to dial %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
+						/* \todo change indicate to something more suitable */
+						sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);		/* will auto hangup after SCCP_HANGUP_TIMEOUT */
+						break;
+					case AST_PBX_CALL_LIMIT:
+						pbx_log(LOG_WARNING, "%s: (sccp_pbx_softswitch) call limit reached for channel %s-%08x failed to start new thread to dial %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
+						sccp_indicate(d, c, SCCP_CHANNELSTATE_CONGESTION);		/* will auto hangup after SCCP_HANGUP_TIMEOUT */
+						break;
+					case AST_PBX_SUCCESS:
+						sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) pbx started\n", DEV_ID_LOG(d));
 #ifdef CS_MANAGER_EVENTS
-					if (GLOB(callevents)) {
-						manager_event(EVENT_FLAG_SYSTEM, "ChannelUpdate", "Channel: %s\r\nUniqueid: %s\r\nChanneltype: %s\r\nSCCPdevice: %s\r\nSCCPline: %s\r\nSCCPcallid: %08x\r\nSCCPCallDesignator: %s\r\n", (pbx_channel) ? pbx_channel_name(pbx_channel) : "(null)", (pbx_channel) ? pbx_channel_uniqueid(pbx_channel) : "(null)", "SCCP", (d) ? DEV_ID_LOG(d) : "(null)", (l && l->name) ? l->name : "(null)", (c && c->callid) ? c->callid : 0, c ? c->designator : "(null)");
-					}
+						if (GLOB(callevents)) {
+							manager_event(EVENT_FLAG_SYSTEM, "ChannelUpdate", "Channel: %s\r\nUniqueid: %s\r\nChanneltype: %s\r\nSCCPdevice: %s\r\nSCCPline: %s\r\nSCCPcallid: %08x\r\nSCCPCallDesignator: %s\r\n", (pbx_channel) ? pbx_channel_name(pbx_channel) : "(null)", (pbx_channel) ? pbx_channel_uniqueid(pbx_channel) : "(null)", "SCCP", (d) ? DEV_ID_LOG(d) : "(null)", (l && l->name) ? l->name : "(null)", (c && c->callid) ? c->callid : 0,
+								      c ? c->designator : "(null)");
+						}
 #endif														// CS_MANAGER_EVENTS
-					break;
+						break;
+				}
+				sccp_device_setLastNumberDialed(d, shortenedNumber);
+			} else {
+				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) pbx_check_hangup(chan): %d on line %s\n", DEV_ID_LOG(d), (pbx_channel && pbx_check_hangup(pbx_channel)), l->name);
 			}
-			sccp_device_setLastNumberDialed(d, shortenedNumber);
 		} else {
-			sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) channel %s-%08x shortenedNumber: %s, pbx_check_hangup(chan): %d, extension exists: %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber, (pbx_channel && pbx_check_hangup(pbx_channel)), (extension_exists != SCCP_EXTENSION_NOTEXISTS) ? "TRUE" : "FALSE");
+			sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) channel %s-%08x shortenedNumber: %s, extension exists: %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber, (extension_exists != SCCP_EXTENSION_NOTEXISTS) ? "TRUE" : "FALSE");
 			pbx_log(LOG_NOTICE, "%s: Call from '%s' to extension '%s', rejected because the extension could not be found in context '%s'\n", DEV_ID_LOG(d), l->name, shortenedNumber, pbx_channel ? pbx_channel_context(pbx_channel) : "pbx_channel==NULL");
 			/* timeout and no extension match */
-			sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);
-			c->scheduler.hangup = sccp_sched_add(15000, sccp_channel_sched_endcall_by_callid, &c->callid);
+			if (pbx_channel && !pbx_check_hangup(pbx_channel)) {
+				pbx_log(LOG_NOTICE, "%s: Scheduling Hangup of Call: %s\n", DEV_ID_LOG(d), c->designator);
+				sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);				/* will auto hangup after SCCP_HANGUP_TIMEOUT */
+			}
 		}
 
 		sccp_log((DEBUGCAT_PBX + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) quit\n", DEV_ID_LOG(d));
@@ -1156,3 +1203,4 @@ int sccp_pbx_transfer(PBX_CHANNEL_TYPE * ast, const char *dest)
 #endif
 
 #endif
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
