@@ -1,12 +1,11 @@
-
 /*!
- * \file	sccp_refcount.c
- * \brief	SCCP Refcount Class
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- * 		See the LICENSE file at the top of the source tree.
+ * \file        sccp_refcount.c
+ * \brief       SCCP Refcount Class
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  * 
- * $Date: 2014-05-30 19:56:13 +0200 (Fri, 30 May 2014) $
- * $Revision: 5449 $  
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $  
  */
 
 /*!
@@ -32,7 +31,7 @@
  *
  * - Rule 4: When releasing an object the pointer we had toward the object should be nullified immediatly, either of these solutions is possible:
  *   \code
- *   d = sccp_device_release(d);		// sccp_release always returns NULL
+ *   d = sccp_device_release(d);                // sccp_release always returns NULL
  *   \endcode
  *   or 
  *   \code
@@ -40,8 +39,8 @@
  *   d = NULL;
  *   \endcode
  *   
- * - Rule 5: 	You <b><em>cannnot</em></b> use free on a refcounted object. Destruction of the refcounted object and subsequent freeing of the occupied memory is performed by the sccp_release 
- *   		when the number of reference reaches 0. To finalize the use of a refcounted object just release the object one final time, to negate the initial refcount of 1 during creation.
+ * - Rule 5:    You <b><em>cannnot</em></b> use free on a refcounted object. Destruction of the refcounted object and subsequent freeing of the occupied memory is performed by the sccp_release 
+ *              when the number of reference reaches 0. To finalize the use of a refcounted object just release the object one final time, to negate the initial refcount of 1 during creation.
  * .
  * These rules need to followed to the letter !
  */
@@ -59,8 +58,8 @@
 #include "sccp_indicate.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5449 $")
-//nb: SCCP_HASH_PRIME defined in config.h, default 536
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+    //nb: SCCP_HASH_PRIME defined in config.h, default 563
 #define SCCP_SIMPLE_HASH(_a) (((unsigned long)(_a)) % SCCP_HASH_PRIME)
 #define SCCP_LIVE_MARKER 13
 #define REF_FILE "/tmp/sccp_refs"
@@ -108,10 +107,20 @@ static struct refcount_objentry {
 	SCCP_RWLIST_HEAD (, RefCountedObject) refCountedObjects;						//!< one rwlock per hash table entry, used to modify list
 } *objects[SCCP_HASH_PRIME];											//!< objects hash table
 
+#if CS_REFCOUNT_DEBUG
+static FILE *sccp_ref_debug_log;
+#endif
+
 void sccp_refcount_init(void)
 {
-	sccp_log((DEBUGCAT_REFCOUNT + DEBUGCAT_HIGH)) ("SCCP: (Refcount) init\n");
+	sccp_log((DEBUGCAT_REFCOUNT + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_1 "SCCP: (Refcount) init\n");
 	pbx_rwlock_init_notracking(&objectslock);								// No tracking to safe cpu cycles
+#if CS_REFCOUNT_DEBUG
+	sccp_ref_debug_log = fopen(REF_FILE, "w");
+	if (!sccp_ref_debug_log) {
+		pbx_log(LOG_NOTICE, "SCCP: Failed to open ref debug log file '%s'\n", REF_FILE);
+	}
+#endif
 	runState = SCCP_REF_RUNNING;
 }
 
@@ -145,12 +154,19 @@ void sccp_refcount_destroy(void)
 			SCCP_RWLIST_UNLOCK(&(objects[x])->refCountedObjects);
 			SCCP_RWLIST_HEAD_DESTROY(&(objects[x])->refCountedObjects);
 
-			sccp_free(objects[x]);                                                                  // free hashtable entry
+			sccp_free(objects[x]);									// free hashtable entry
 			objects[x] = NULL;
 		}
 	}
 	ast_rwlock_unlock(&objectslock);
 	pbx_rwlock_destroy(&objectslock);
+#if CS_REFCOUNT_DEBUG
+	if (sccp_ref_debug_log) {
+		fclose(sccp_ref_debug_log);
+		sccp_ref_debug_log = NULL;
+		pbx_log(LOG_NOTICE, "SCCP: ref debug log file: %s closed\n", REF_FILE);
+	}
+#endif
 	runState = SCCP_REF_DESTROYED;
 }
 
@@ -171,7 +187,12 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 	void *ptr = NULL;
 	int hash;
 
-	if (!(obj = calloc(1, size + sizeof(*obj)))) {
+	if (!runState) {
+		ast_log(LOG_ERROR, "SCCP: (sccp_refcount_object_alloc) Not Running Yet!\n");
+		return NULL;
+	}
+
+	if (!(obj = sccp_calloc(1, size + sizeof(*obj)))) {
 		ast_log(LOG_ERROR, "SCCP: (sccp_refcount_object_alloc) Memory allocation failure (obj)");
 		return NULL;
 	}
@@ -196,9 +217,9 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 		// create new hashtable head when necessary (should this possibly be moved to refcount_init, to avoid raceconditions ?)
 		ast_rwlock_wrlock(&objectslock);
 		if (!objects[hash]) {										// check again after getting the lock, to see if another thread did not create the head already
-			if (!(objects[hash] = malloc(sizeof(struct refcount_objentry)))) {
+			if (!(objects[hash] = sccp_malloc(sizeof(struct refcount_objentry)))) {
 				ast_log(LOG_ERROR, "SCCP: (sccp_refcount_object_alloc) Memory allocation failure (hashtable)");
-				free(obj);
+				sccp_free(obj);
 				obj = NULL;
 				ast_rwlock_unlock(&objectslock);
 				return NULL;
@@ -216,11 +237,9 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 	obj->alive = SCCP_LIVE_MARKER;
 
 #if CS_REFCOUNT_DEBUG
-	FILE *refo = fopen(REF_FILE, "a");
-
-	if (refo) {
-		fprintf(refo, "%p =1   %s:%d:%s (allocate new %s, len: %d, destructor: %p) [%p]\n", obj, __FILE__, __LINE__, __PRETTY_FUNCTION__, (&obj_info[obj->type])->datatype, (int) obj->len, (&obj_info[type])->destructor, ptr);
-		fclose(refo);
+	if (sccp_ref_debug_log) {
+		fprintf(sccp_ref_debug_log, "%p,+1,%d,%s,%d,%s,**constructor**,%s:%s\n", ptr, ast_get_tid(), __FILE__, __LINE__, __PRETTY_FUNCTION__, (&obj_info[obj->type])->datatype, obj->identifier);
+		fflush(sccp_ref_debug_log);
 	}
 #endif
 	memset(ptr, 0, size);
@@ -230,54 +249,37 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 #if CS_REFCOUNT_DEBUG
 int __sccp_refcount_debug(void *ptr, RefCountedObject * obj, int delta, const char *file, int line, const char *func)
 {
-	if (ptr == NULL) {
-		FILE *refo = fopen(REF_FILE, "a");
+	if (!sccp_ref_debug_log) {
+		return -1;
+	}
 
-		if (refo) {
-			fprintf(refo, "%p **PTR IS NULL !!** %s:%d:%s\n", ptr, file, line, func);
-			fclose(refo);
-		}
+	if (ptr == NULL) {
+		fprintf(sccp_ref_debug_log, "%p **PTR IS NULL !!** %s:%d:%s\n", ptr, file, line, func);
 		// ast_assert(0);
+		fflush(sccp_ref_debug_log);
 		return -1;
 	}
 	if (obj == NULL) {
-		FILE *refo = fopen(REF_FILE, "a");
-
-		if (refo) {
-			fprintf(refo, "%p **OBJ ALREADY DESTROYED !!** %s:%d:%s\n", ptr, file, line, func);
-			fclose(refo);
-		}
+		fprintf(sccp_ref_debug_log, "%p **OBJ ALREADY DESTROYED !!** %s:%d:%s\n", ptr, file, line, func);
 		// ast_assert(0);
+		fflush(sccp_ref_debug_log);
 		return -1;
 	}
 
 	if (delta == 0 && obj->alive != SCCP_LIVE_MARKER) {
-		FILE *refo = fopen(REF_FILE, "a");
-
-		if (refo) {
-			fprintf(refo, "%p **OBJ Already destroyed and Declared DEAD !!** %s:%d:%s (%s:%s) [@%d] [%p]\n", ptr, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier, obj->refcount, ptr);
-			fclose(refo);
-		}
+		fprintf(sccp_ref_debug_log, "%p **OBJ Already destroyed and Declared DEAD !!** %s:%d:%s (%s:%s) [@%d] [%p]\n", ptr, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier, obj->refcount, ptr);
 		// ast_assert(0);
+		fflush(sccp_ref_debug_log);
 		return -1;
 	}
 
 	if (delta != 0) {
-		FILE *refo = fopen(REF_FILE, "a");
-
-		if (refo) {
-			fprintf(refo, "%p %s%d   %s:%d:%s (%s:%s) [@%d] [%p]\n", obj, (delta < 0 ? "" : "+"), delta, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier, obj->refcount, ptr);
-			fclose(refo);
-		}
+		fprintf(sccp_ref_debug_log, "%p,%s%d,%d,%s,%d,%s,%d,%s:%s\n", ptr, (delta < 0 ? "" : "+"), delta, ast_get_tid(), file, line, func, obj ? obj->refcount : -1, (&obj_info[obj->type])->datatype, obj->identifier);
 	}
 	if (obj->refcount + delta == 0 && (&obj_info[obj->type])->destructor != NULL) {
-		FILE *refo = fopen(REF_FILE, "a");
-
-		if (refo) {
-			fprintf(refo, "%p **call destructor** %s:%d:%s (%s:%s)\n", ptr, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier);
-			fclose(refo);
-		}
+		fprintf(sccp_ref_debug_log, "%p,%d,%d,%s,%d,%s,**destructor**,%s:%s\n", ptr, delta, ast_get_tid(), file, line, func, (&obj_info[obj->type])->datatype, obj->identifier);
 	}
+	fflush(sccp_ref_debug_log);
 	return 0;
 }
 #endif
@@ -379,22 +381,22 @@ void sccp_refcount_print_hashtable(int fd)
 				fillfactor++;
 			}
 			if (maxdepth < SCCP_RWLIST_GETSIZE(&(objects[x])->refCountedObjects)) {
-				maxdepth=SCCP_RWLIST_GETSIZE(&(objects[x])->refCountedObjects);
+				maxdepth = SCCP_RWLIST_GETSIZE(&(objects[x])->refCountedObjects);
 			}
 			SCCP_RWLIST_UNLOCK(&(objects[x])->refCountedObjects);
 		}
 	}
 	ast_rwlock_unlock(&objectslock);
 	pbx_cli(fd, "+==============================================================================================+\n");
-	pbx_cli(fd, "| fillfactor = (%03d / %03d) = %02.2f, maxdepth = %02d                                               |\n", fillfactor, SCCP_HASH_PRIME, (float)fillfactor/SCCP_HASH_PRIME, maxdepth);
+	pbx_cli(fd, "| fillfactor = (%03d / %03d) = %02.2f, maxdepth = %02d                                               |\n", fillfactor, SCCP_HASH_PRIME, (float) fillfactor / SCCP_HASH_PRIME, maxdepth);
 	pbx_cli(fd, "+==============================================================================================+\n");
-	
+
 }
 
 #ifdef CS_EXPERIMENTAL
 int sccp_refcount_force_release(long findobj, char *identifier)
 {
-	int x= 0;
+	int x = 0;
 	RefCountedObject *obj = NULL;
 	void *ptr = NULL;
 
@@ -403,7 +405,7 @@ int sccp_refcount_force_release(long findobj, char *identifier)
 		if (objects[x] && &(objects[x])->refCountedObjects) {
 			SCCP_RWLIST_RDLOCK(&(objects[x])->refCountedObjects);
 			SCCP_RWLIST_TRAVERSE(&(objects[x])->refCountedObjects, obj, list) {
-				if (sccp_strequals(obj->identifier, identifier) && (long)obj == findobj) {
+				if (sccp_strequals(obj->identifier, identifier) && (long) obj == findobj) {
 					ptr = obj->data;
 				}
 			}
@@ -492,22 +494,24 @@ gcc_inline void *sccp_refcount_release(const void *ptr, const char *filename, in
 	return NULL;
 }
 
-gcc_inline void sccp_refcount_replace(void **replaceptr, void *newptr, const char *filename, int lineno, const char *func) {
-	if (&newptr == replaceptr) {					// nothing changed
+gcc_inline void sccp_refcount_replace(void **replaceptr, void *newptr, const char *filename, int lineno, const char *func)
+{
+	if ((!replaceptr && !newptr) || (&newptr == replaceptr)) {						// nothing changed
 		return;
 	}
 
-	void *tmpNewPtr = NULL;						// retain new one first
+	void *tmpNewPtr = NULL;											// retain new one first
 	void *oldPtr = *replaceptr;
+
 	if (newptr) {
 		if ((tmpNewPtr = sccp_refcount_retain(newptr, filename, lineno, func))) {
 			*replaceptr = tmpNewPtr;
-			if (oldPtr) { 							// release previous one after
+			if (oldPtr) {										// release previous one after
 				sccp_refcount_release(oldPtr, filename, lineno, func);
 			}
 		}
 	} else {
-		if (oldPtr) { 							// release previous one after
+		if (oldPtr) {											// release previous one after
 			*replaceptr = sccp_refcount_release(oldPtr, filename, lineno, func);
 		}
 	}
@@ -518,9 +522,11 @@ gcc_inline void sccp_refcount_replace(void **replaceptr, void *newptr, const cha
  * Used together with the cleanup attribute, to handle the automatic reference release of an object when we leave the scope in which the 
  * reference was defined. 
  */
-void sccp_refcount_autorelease(void *ptr) 
+void sccp_refcount_autorelease(void *ptr)
 {
 	if (*(void **) ptr) {
 		sccp_refcount_release(*(void **) ptr, __FILE__, __LINE__, __PRETTY_FUNCTION__);
 	}
 }
+
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
